@@ -153,6 +153,10 @@ make_mpeg_decoder_pipeline (const gchar *path, GstIndex *index)
   audio_bin = gst_bin_new ("audio_bin");
   audio_decoder = gst_element_factory_make ("mad", "audio_decoder");
 
+  setup_dynamic_linking (pipeline, demux, "audio_00", 
+		         gst_element_get_pad (audio_decoder, "sink"),
+			 audio_bin, index);
+
   gst_bin_add (GST_BIN (audio_bin), audio_decoder);
 
   if (index) {
@@ -211,7 +215,7 @@ main (gint argc, gchar *argv[])
       POPT_TABLEEND
     };
 
-  if (!gst_init_with_popt_table (&argc, &argv, options) || argc < 3) {
+  if (!gst_init_check_with_popt_table (&argc, &argv, options) || argc < 3) {
     g_print ("usage: %s [-v] <type> <filename>  \n" 
 	     "  type can be: 0 mpeg_systems\n"
 	     "               1 mpeg_decoder\n"
@@ -238,7 +242,7 @@ main (gint argc, gchar *argv[])
       pipeline = make_mpeg_decoder_pipeline (argv[2], index);
       break;
     default:
-      g_print ("unkown type %d\n", atoi (argv[1]));
+      g_print ("unknown type %d\n", atoi (argv[1]));
       return -1;
   }
 
