@@ -171,7 +171,7 @@ gst_alsa_mixer_dispose (GObject * object)
 {
   GstAlsaMixer *mixer = GST_ALSA_MIXER (object);
 
-  if (((gint) mixer->mixer_handle) != -1) return;
+  if (((gint) mixer->mixer_handle) == -1) return;
 
   snd_mixer_close (mixer->mixer_handle);
   mixer->mixer_handle = (snd_mixer_t *) -1;
@@ -188,6 +188,8 @@ gst_alsa_interface_init (GstImplementsInterfaceClass *klass)
 static void
 gst_alsa_mixer_interface_init (GstMixerClass *klass)
 {
+  GST_MIXER_TYPE (klass) = GST_MIXER_HARDWARE;
+  
   /* set up the interface hooks */
   klass->list_tracks = gst_alsa_mixer_list_tracks;
   klass->set_volume = gst_alsa_mixer_set_volume;
@@ -209,6 +211,7 @@ gst_alsa_mixer_build_list (GstAlsaMixer *mixer)
 {
   gint i, count;
   snd_mixer_elem_t *element;
+  GstMixerTrack *track;
 
   g_return_if_fail (((gint) mixer->mixer_handle) != -1);
 
@@ -230,7 +233,7 @@ gst_alsa_mixer_build_list (GstAlsaMixer *mixer)
       while (snd_mixer_selem_has_capture_channel(element, channels))
         channels++;
 
-      GstMixerTrack *track = gst_alsa_mixer_track_new
+      track = gst_alsa_mixer_track_new
         (element, i, channels, GST_MIXER_TRACK_INPUT);
       mixer->tracklist = g_list_append (mixer->tracklist, track);
     }
@@ -245,7 +248,7 @@ gst_alsa_mixer_build_list (GstAlsaMixer *mixer)
       while (snd_mixer_selem_has_playback_channel(element, channels))
         channels++;
 
-      GstMixerTrack *track = gst_alsa_mixer_track_new
+      track = gst_alsa_mixer_track_new
         (element, i, channels, GST_MIXER_TRACK_OUTPUT);
       mixer->tracklist = g_list_append (mixer->tracklist, track);
     }
