@@ -24,19 +24,21 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+
+#include "gst/gst-i18n-plugin.h"
 #include <gst/gst.h>
 #include <gst/audio/audio.h>
+#include <string.h>
+#include <errno.h>
+
 #include "gstafsrc.h"
 
 /* elementfactory information */
 static GstElementDetails afsrc_details = {
   "Audiofile Src",
   "Source/Audio",
-  "LGPL",
   "Read audio files from disk using libaudiofile",
-  VERSION,
   "Thomas <thomas@apestaart.org>",
-  "(C) 2001"
 };
 
 
@@ -54,22 +56,19 @@ enum {
 
 /* added a src factory function to force audio/raw MIME type */
 /* I think the caps can be broader, we need to change that somehow */
-GST_PAD_TEMPLATE_FACTORY (afsrc_src_factory,
+static GstStaticPadTemplate afsrc_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (  
-    "audiofile_src",
-    "audio/x-raw-int",
-        "endianness",       GST_PROPS_INT (G_BYTE_ORDER),
-        "signed",           GST_PROPS_LIST (
-				  GST_PROPS_BOOLEAN (TRUE),
-       				  GST_PROPS_BOOLEAN (FALSE)
-       			    ),
-        "width",            GST_PROPS_INT_RANGE (8, 16),
-        "depth",            GST_PROPS_INT_RANGE (8, 16),
-        "rate",             GST_PROPS_INT_RANGE (4000, 48000), /*FIXME*/
-        "channels",         GST_PROPS_INT_RANGE (1, 2)
+  GST_STATIC_CAPS ("audio/x-raw-int, "
+    "rate = (int) [ 1, MAX ], "
+    "channels = (int) [ 1, MAX ], "
+    "endianness = (int) BYTE_ORDER, "
+    "width = (int) { 8, 16 }, "
+    "depth = (int) { 8, 16 }, "
+    "signed = (boolean) { true, false }, "
+    "buffer-frames = (int) [ 1, MAX ]"
   )
 );
 
@@ -99,6 +98,7 @@ gst_afsrc_types_get_type (void)
   return afsrc_types_type;
 }
 */
+static void             gst_afsrc_base_init             (gpointer g_class);
 static void		gst_afsrc_class_init		(GstAFSrcClass *klass);
 static void		gst_afsrc_init			(GstAFSrc *afsrc);
 
@@ -124,7 +124,8 @@ gst_afsrc_get_type (void)
 
   if (!afsrc_type) {
     static const GTypeInfo afsrc_info = {
-      sizeof (GstAFSrcClass),      NULL,
+      sizeof (GstAFSrcClass),
+      gst_afsrc_base_init,
       NULL,
       (GClassInitFunc) gst_afsrc_class_init,
       NULL,
@@ -138,6 +139,16 @@ gst_afsrc_get_type (void)
   return afsrc_type;
 }
 
+static void
+gst_afsrc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&afsrc_src_factory));
+  gst_element_class_set_details (element_class, &afsrc_details);
+}
+
 static void
 gst_afsrc_class_init (GstAFSrcClass *klass) 
 {
@@ -170,8 +181,10 @@ static void
 gst_afsrc_init (GstAFSrc *afsrc) 
 {
   /* no need for a template, caps are set based on file, right ? */
-  afsrc->srcpad = gst_pad_new_from_template (afsrc_src_factory (), "src");
+  afsrc->srcpad = gst_pad_new_from_template (
+      gst_element_get_pad_template (GST_ELEMENT (afsrc), "src"), "src");
   gst_element_add_pad (GST_ELEMENT (afsrc), afsrc->srcpad);
+  gst_pad_use_explicit_caps (afsrc->srcpad);
   gst_pad_set_get_function (afsrc->srcpad, gst_afsrc_get);
 
   afsrc->bytes_per_read = 4096;
@@ -274,22 +287,20 @@ gst_afsrc_get_property (GObject *object, guint prop_id, GValue *value, GParamSpe
 }
 
 gboolean
-gst_afsrc_plugin_init (GModule *module, GstPlugin *plugin)
+gst_afsrc_plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  
-  factory = gst_element_factory_new ("afsrc", GST_TYPE_AFSRC,
-                                    &afsrc_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (afsrc_src_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
   /* load audio support library */
   if (!gst_library_load ("gstaudio"))
     return FALSE;
-  
+
+  if (!gst_element_register (plugin, "afsrc", GST_RANK_NONE, GST_TYPE_AFSRC))
+    return FALSE;
+
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return TRUE;
 }
 
@@ -304,10 +315,9 @@ gst_afsrc_open_file (GstAFSrc *src)
   src->file = afOpenFile (src->filename, "r", AF_NULL_FILESETUP);
   if (src->file == AF_NULL_FILEHANDLE)
   {
-    g_print ("ERROR: gstafsrc: Could not open file %s for reading\n",
-		src->filename);
-    gst_element_error (GST_ELEMENT (src), g_strconcat ("opening file \"",
-			src->filename, "\"", NULL));
+     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+                        (_("Could not open file \"%s\" for reading."), src->filename),
+                        ("system error: %s", strerror (errno)));
     return FALSE;
   }
 
@@ -339,19 +349,15 @@ gst_afsrc_open_file (GstAFSrc *src)
   }
   
   /* set caps on src */
-  /*FIXME: add all the possible formats, especially float ! */ 
-  gst_pad_try_set_caps (src->srcpad, 
-		  GST_CAPS_NEW (
-    		    "af_src",
-                    "audio/x-raw-int",
-                      "endianness", GST_PROPS_INT (G_BYTE_ORDER),   /*FIXME */
-                      "signed",     GST_PROPS_BOOLEAN (src->is_signed),
-                      "width",      GST_PROPS_INT (src->width),
-                      "depth",      GST_PROPS_INT (src->width),
-                      "rate",       GST_PROPS_INT (src->rate),
-                      "channels",   GST_PROPS_INT (src->channels)
-        	   )
-                 );
+  gst_pad_set_explicit_caps (src->srcpad, 
+      gst_caps_new_simple ("audio/x-raw-int",
+        "endianness", G_TYPE_INT, G_BYTE_ORDER,
+        "signed",     G_TYPE_BOOLEAN, src->is_signed,
+        "width",      G_TYPE_INT, src->width,
+        "depth",      G_TYPE_INT, src->width,
+        "rate",       G_TYPE_INT, src->rate,
+        "channels",   G_TYPE_INT, src->channels,
+        NULL));
 
   GST_FLAG_SET (src, GST_AFSRC_OPEN);
 
@@ -367,11 +373,10 @@ gst_afsrc_close_file (GstAFSrc *src)
 /*  if (fclose (src->file) != 0) 	*/
   if (afCloseFile (src->file) != 0)
   {
-    g_print ("WARNING: afsrc: oops, error closing !\n");
-    perror ("close");
-    gst_element_error (GST_ELEMENT (src), g_strconcat("closing file \"", src->filename, "\"", NULL));
-  }
-  else {
+    GST_ELEMENT_ERROR (src, RESOURCE, CLOSE,
+                       (_("Error closing file \"%s\"."), src->filename),
+                       GST_ERROR_SYSTEM);
+  } else {
     GST_FLAG_UNSET (src, GST_AFSRC_OPEN);
   }
 }
