@@ -35,6 +35,7 @@
 #include <sys/types.h>
 #include <dirent.h>
 #include <errno.h>
+#include "gst/gst-i18n-plugin.h"
 
 /* taken from linux/cdrom.h */
 #define CD_MSF_OFFSET       150	/* MSF numbering offset of first frame */
@@ -51,21 +52,21 @@ static GstElementDetails cdparanoia_details = {
   "Erik Walthinsen <omega@cse.ogi.edu>",
 };
 
-GST_PAD_TEMPLATE_FACTORY (cdparanoia_src_factory,
-  "src",
-  GST_PAD_SRC,
-  GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "cdparanoia_src",
-    "audio/x-raw-int",
-	"endianness", 	GST_PROPS_INT (G_BYTE_ORDER),
-	"signed", 	GST_PROPS_BOOLEAN (TRUE),
-	"width", 	GST_PROPS_INT (16),
-	"depth", 	GST_PROPS_INT (16),
-	"rate", 	GST_PROPS_INT (44100),
-	"channels", 	GST_PROPS_INT (2),
-	"chunksize", 	GST_PROPS_INT (CD_FRAMESIZE_RAW)
-  )
+static GstStaticPadTemplate cdparanoia_src_template =
+  GST_STATIC_PAD_TEMPLATE (
+    "src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (
+      "audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) true, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) 44100, "
+      "channels = (int) 2, "
+      "chunksize = (int) " G_STRINGIFY(CD_FRAMESIZE_RAW)
+    )
 );
 
 
@@ -202,8 +203,10 @@ cdparanoia_base_init (gpointer g_class)
 {
 	GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-	gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (cdparanoia_src_factory));
+	gst_element_class_add_pad_template (element_class, 
+	  gst_static_pad_template_get (&cdparanoia_src_template));
 	gst_element_class_set_details (element_class, &cdparanoia_details);
+
 }
 
 static void
@@ -221,15 +224,15 @@ cdparanoia_class_init (CDParanoiaClass *klass)
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
   cdparanoia_signals[SMILIE_CHANGE] =
-    g_signal_new ("smilie_change", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+    g_signal_new ("smilie-change", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (CDParanoiaClass, smilie_change), NULL, NULL,
 		  g_cclosure_marshal_VOID__STRING, G_TYPE_NONE, 1, G_TYPE_STRING);
   cdparanoia_signals[TRANSPORT_ERROR] =
-    g_signal_new ("transport_error", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+    g_signal_new ("transport-error", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (CDParanoiaClass, transport_error), NULL, NULL,
 		  g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
   cdparanoia_signals[UNCORRECTED_ERROR] =
-    g_signal_new ("uncorrected_error", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
+    g_signal_new ("uncorrected-error", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (CDParanoiaClass, uncorrected_error), NULL, NULL,
 		  g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
 
@@ -289,7 +292,8 @@ static void
 cdparanoia_init (CDParanoia *cdparanoia)
 {
   cdparanoia->srcpad =
-    gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (cdparanoia_src_factory), "src");
+    gst_pad_new_from_template (
+      gst_static_pad_template_get (&cdparanoia_src_template), "src");
   gst_pad_set_get_function (cdparanoia->srcpad, cdparanoia_get);
   gst_pad_set_event_function (cdparanoia->srcpad, cdparanoia_event);
   gst_pad_set_event_mask_function (cdparanoia->srcpad, cdparanoia_get_event_mask);
@@ -630,7 +634,9 @@ cdparanoia_open (CDParanoia *src)
 
   /* fail if the device couldn't be found */
   if (src->d == NULL) {
-    GST_DEBUG ("couldn't open device");
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+                       (_("Could not open CD device for reading.")),
+                       ("cdda_identify failed"));
     return FALSE;
   }
 
@@ -645,7 +651,9 @@ cdparanoia_open (CDParanoia *src)
 
   /* open the disc */
   if (cdda_open (src->d)) {
-    GST_DEBUG ("couldn't open disc");
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ,
+                       (_("Could not open CD device for reading.")),
+                       ("cdda_open failed"));
     cdda_close (src->d);
     src->d = NULL;
     return FALSE;
@@ -684,7 +692,7 @@ cdparanoia_open (CDParanoia *src)
   /* create the paranoia struct and set it up */
   src->p = paranoia_init (src->d);
   if (src->p == NULL) {
-    GST_DEBUG ("couldn't create paranoia struct");
+    GST_ELEMENT_ERROR (src, LIBRARY, INIT, (NULL), (NULL));
     return FALSE;
   }
 
@@ -798,9 +806,9 @@ cdparanoia_event (GstPad *pad, GstEvent *event)
   src = CDPARANOIA (gst_pad_get_parent (pad));
 
   if (!GST_FLAG_IS_SET (src, CDPARANOIA_OPEN)) {
-		g_print ("Not open\n");
+    GST_DEBUG ("device not open, cannot handle event");
     goto error;
-	}
+  }
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
@@ -861,7 +869,7 @@ cdparanoia_event (GstPad *pad, GstEvent *event)
         if (paranoia_seek (src->p, seg_start_sector, SEEK_SET) > -1) {
 					GST_DEBUG ("seeked to %" G_GINT64_FORMAT, seg_start_sector);
 					
-					src->segment_start_sector = seg_start_sector;;
+					src->segment_start_sector = seg_start_sector;
 					src->cur_sector = src->segment_start_sector;
 				}
 				else {
@@ -871,7 +879,7 @@ cdparanoia_event (GstPad *pad, GstEvent *event)
       if (seg_end_sector != -1) {
         seg_end_sector = CLAMP (seg_end_sector, 
 																src->first_sector, src->last_sector);
-        src->segment_end_sector = seg_end_sector;;
+        src->segment_end_sector = seg_end_sector;
       }
       GST_DEBUG ("configured for %d -> %d sectors\n", 
 								 src->segment_start_sector, 
