@@ -32,6 +32,7 @@
 
 #define NAS_SOUND_PORT_DURATION	(2)
 
+GST_DEBUG_CATEGORY(NAS);
 /* Signals and args */
 enum {
   /* FILL ME */
@@ -55,13 +56,13 @@ GST_STATIC_PAD_TEMPLATE (
       "signed = (boolean) TRUE, "
       "width = (int) 16, "
       "depth = (int) 16, "
-      "rate = (int) [ 8000, 96000 ], "
+      "rate = (int) [ 1000, 96000 ], "
       "channels = (int) [ 1, 2 ]; "
     "audio/x-raw-int, "
       "signed = (boolean) FALSE, "
       "width = (int) 8, "
       "depth = (int) 8, "
-      "rate = (int) [ 8000, 96000 ], "
+      "rate = (int) [ 1000, 96000 ], "
       "channels = (int) [ 1, 2 ]"
   )
 );
@@ -73,6 +74,7 @@ static void			gst_nassink_init		(GstNassink *nassink);
 static gboolean			gst_nassink_open_audio		(GstNassink *sink);
 static void			gst_nassink_close_audio		(GstNassink *sink);
 static GstElementStateReturn	gst_nassink_change_state	(GstElement *element);
+static GstCaps*                 gst_nassink_getcaps             (GstPad *pad);
 static gboolean			gst_nassink_sync_parms		(GstNassink *nassink);
 static GstPadLinkReturn		gst_nassink_sinkconnect		(GstPad *pad, const GstCaps *caps);
 
@@ -130,6 +132,7 @@ gst_nassink_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sink_factory));
   gst_element_class_set_details (element_class, &nassink_details);
+  GST_DEBUG_CATEGORY_INIT(NAS, "NAS", 0, NULL);
 }
 
 static void
@@ -160,11 +163,13 @@ gst_nassink_class_init (GstNassinkClass *klass)
 static void
 gst_nassink_init(GstNassink *nassink)
 {
+  GST_CAT_DEBUG(NAS,"nassink: init");
   nassink->sinkpad = gst_pad_new_from_template (
       gst_static_pad_template_get (&sink_factory), "sink");
   gst_element_add_pad(GST_ELEMENT(nassink), nassink->sinkpad);
   gst_pad_set_chain_function(nassink->sinkpad, GST_DEBUG_FUNCPTR(gst_nassink_chain));
   gst_pad_set_link_function(nassink->sinkpad, gst_nassink_sinkconnect);
+  gst_pad_set_getcaps_function(nassink->sinkpad, gst_nassink_getcaps);
 
   nassink->mute = FALSE;
   nassink->depth = 16;
@@ -181,6 +186,31 @@ gst_nassink_init(GstNassink *nassink)
   nassink->buf = NULL;
 }
 
+static GstCaps*
+gst_nassink_getcaps (GstPad *pad)
+{
+  GstNassink *nassink = GST_NASSINK(gst_pad_get_parent(pad));
+  GstCaps *templatecaps = gst_caps_copy(gst_pad_get_pad_template_caps(pad));
+  GstCaps *caps;
+  int i;
+  AuServer *server;
+
+  server = AuOpenServer(nassink->host, 0, NULL, 0, NULL, NULL);
+  if (!server)
+    return templatecaps;
+  
+  for (i = 0; i < gst_caps_get_size (templatecaps); i++) {
+    GstStructure *structure = gst_caps_get_structure (templatecaps, i);
+
+    gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, AuServerMinSampleRate(server), AuServerMaxSampleRate(server), NULL);
+  }
+  caps = gst_caps_intersect(templatecaps, gst_pad_get_pad_template_caps(pad));
+  gst_caps_free(templatecaps);
+
+  return caps;
+  
+}
+
 static gboolean
 gst_nassink_sync_parms (GstNassink *nassink)
 {
@@ -191,8 +221,10 @@ gst_nassink_sync_parms (GstNassink *nassink)
 
   if (nassink->audio == NULL) return TRUE;
 
+  GST_CAT_DEBUG(NAS,"depth=%i rate=%i", nassink->depth, nassink->rate);
   if (nassink->flow != AuNone)
   {
+    GST_CAT_DEBUG(NAS,"flushing buffer");
     while (nassink->pos && nassink->buf)
       NAS_flush(nassink);
     AuStopFlow( nassink->audio, nassink->flow, NULL);
@@ -253,7 +285,6 @@ gst_nassink_chain (GstPad *pad, GstData *_data)
 
   if (GST_BUFFER_DATA (buf) != NULL) {
     if (!nassink->mute && nassink->audio != NULL) {
-      GST_DEBUG ("nassink: data=%p size=%d", GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
 
       remaining = GST_BUFFER_SIZE (buf);
       while ((nassink->flow != AuNone) && ( remaining > 0)) {
@@ -369,6 +400,11 @@ gst_nassink_open_audio (GstNassink *sink)
   sink->audio = AuOpenServer(sink->host, 0, NULL, 0, NULL, NULL);
   if (sink->audio == NULL)
     return FALSE;
+  sink->device = NAS_getDevice(sink->audio, sink->tracks);
+  if (sink->device == AuNone) {
+    GST_CAT_DEBUG(NAS,"no device with %i tracks found", sink->tracks);
+    return FALSE;
+  }
 
   sink->flow = AuNone;
   sink->size = 0;
@@ -379,6 +415,7 @@ gst_nassink_open_audio (GstNassink *sink)
 
   GST_FLAG_SET (sink, GST_NASSINK_OPEN);
 
+  GST_CAT_DEBUG(NAS,"opened audio device");
   return TRUE;
 }
 
@@ -408,7 +445,7 @@ gst_nassink_close_audio (GstNassink *sink)
 
   GST_FLAG_UNSET (sink, GST_NASSINK_OPEN);
 
-  GST_DEBUG ("nassink: closed sound device");
+  GST_CAT_DEBUG (NAS,"closed audio device");
 }
 
 static GstElementStateReturn
@@ -578,22 +615,60 @@ NAS_allocBuffer(GstNassink *sink)
 static int
 NAS_createFlow(GstNassink *sink, unsigned char format, unsigned short rate, int numTracks)
 {
-  AuDeviceID device;
   AuElement elements[2];
   AuUint32 buf_samples;
 
-  device = NAS_getDevice(sink->audio, numTracks);
-  if (device == AuNone) {
+  sink->flow = AuGetScratchFlow(sink->audio, NULL);
+  if (sink->flow == 0) {
+    GST_CAT_DEBUG(NAS,"couldn't get flow");
     return -1;
   }
 
-  sink->flow = AuGetScratchFlow(sink->audio, NULL);
-  if (sink->flow == 0) {
-    return -1;
+  /* free old Elements and reconnet to server, needed to change samplerate */
+  {
+    AuBool clocked;
+    int num_elements;
+    AuStatus status;
+    AuElement *oldelems;
+    oldelems = AuGetElements(sink->audio, sink->flow, &clocked, &num_elements, &status);
+    if (num_elements > 0) {
+      GST_CAT_DEBUG(NAS,"GetElements status: %i", status);
+      if (oldelems)
+        AuFreeElements(sink->audio, num_elements, oldelems);
+      gst_nassink_close_audio(sink);
+      gst_nassink_open_audio(sink);
+      sink->flow = AuGetScratchFlow(sink->audio, NULL);
+      if (sink->flow == 0) {
+        GST_CAT_DEBUG(NAS,"couldn't get flow");
+        return -1;
+      }
+    }
+  }
+
+  /* free old Elements and reconnet to server, needed to change samplerate */
+  {
+    AuBool clocked;
+    int num_elements;
+    AuStatus status;
+    AuElement *oldelems;
+    oldelems = AuGetElements(sink->audio, sink->flow, &clocked, &num_elements, &status);
+    if (num_elements > 0) {
+      GST_CAT_DEBUG(NAS,"GetElements status: %i", status);
+      if (oldelems)
+        AuFreeElements(sink->audio, num_elements, oldelems);
+      gst_nassink_close_audio(sink);
+      gst_nassink_open_audio(sink);
+      sink->flow = AuGetScratchFlow(sink->audio, NULL);
+      if (sink->flow == 0) {
+        GST_CAT_DEBUG(NAS,"couldn't get flow");
+        return -1;
+      }
+    }
   }
 
   buf_samples = rate * NAS_SOUND_PORT_DURATION;
 
+
   AuMakeElementImportClient( &elements[0],		/* element */
                              rate,			/* rate */
                              format,			/* format */
@@ -608,7 +683,7 @@ NAS_createFlow(GstNassink *sink, unsigned char format, unsigned short rate, int
 
   AuMakeElementExportDevice( &elements[1],		/* element */
                              0,				/* input */
-                             device,			/* device */
+                             sink->device,		/* device */
                              rate,			/* rate */
                              AuUnlimitedSamples,	/* num samples */
                              0,				/* num actions */
