@@ -272,6 +272,10 @@ gst_ogg_get_pad_by_pad (GstOggDemux *ogg, GstPad *pad)
   GSList *walk;
   GstOggPad *cur;
   
+  if (ogg->current_chain == -1) {
+    GST_DEBUG_OBJECT (ogg, "no active chain, returning NULL");
+    return NULL;
+  }
   for (walk = CURRENT_CHAIN (ogg)->pads; walk; walk = g_slist_next (walk)) {
     cur = (GstOggPad *) walk->data;
     if (cur->pad == pad)
@@ -288,6 +292,9 @@ gst_ogg_demux_src_query (GstPad *pad, GstQueryType type,
   GstOggDemux *ogg = GST_OGG_DEMUX (gst_pad_get_parent (pad));
   GstOggPad *cur = gst_ogg_get_pad_by_pad (ogg, pad);
 
+  if (!cur)
+    return FALSE;
+
   switch (type) {
     case GST_QUERY_TOTAL: {
       if (*format == GST_FORMAT_DEFAULT) {
@@ -592,6 +599,7 @@ static GstOggPad *
 gst_ogg_pad_new (GstOggDemux *ogg, int serial)
 {
   GstOggPad *ret = g_new0 (GstOggPad, 1);
+  GstTagList *list = gst_tag_list_new ();
 
   ret->serial = serial;
   if (ogg_stream_init (&ret->stream, serial) != 0) {
@@ -599,6 +607,8 @@ gst_ogg_pad_new (GstOggDemux *ogg, int serial)
     g_free (ret);
     return NULL;
   }
+  gst_tag_list_add (list, GST_TAG_MERGE_REPLACE, GST_TAG_SERIAL, serial, NULL);
+  gst_element_found_tags (GST_ELEMENT (ogg), list);
   GST_LOG_OBJECT (ogg, "created new ogg src %p for stream with serial %d", ret, serial);
 
   return ret;
