@@ -22,6 +22,7 @@
 #include "config.h"
 #endif
 
+#include "gst/gst-i18n-plugin.h"
 #include <string.h>
 #include <gst/gst.h>
 
@@ -53,26 +54,20 @@ enum {
   ARG_CREATE_PADS
 };
 
-GST_PAD_TEMPLATE_FACTORY (sf_src_factory,
+static GstStaticPadTemplate sf_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src%d",
   GST_PAD_SRC,
   GST_PAD_REQUEST,
-  gst_caps_new (
-    "sf_src",
-    "audio/x-raw-float",
-    GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS
-  )
+  GST_STATIC_CAPS (GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS)
 );
 
-GST_PAD_TEMPLATE_FACTORY (sf_sink_factory,
+static GstStaticPadTemplate sf_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink%d",
   GST_PAD_SINK,
   GST_PAD_REQUEST,
-  gst_caps_new (
-    "sf_sink",
-    "audio/x-raw-float",
-    GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_PROPS
-  )
+  GST_STATIC_CAPS (GST_AUDIO_FLOAT_STANDARD_PAD_TEMPLATE_CAPS)
 );
 
 #define GST_TYPE_SF_MAJOR_TYPES (gst_sf_major_types_get_type())
@@ -162,7 +157,7 @@ static GstPad*		gst_sf_request_new_pad	(GstElement *element, GstPadTemplate *tem
 static void		gst_sf_release_request_pad (GstElement *element, GstPad *pad);
 static GstElementStateReturn gst_sf_change_state (GstElement *element);
 
-static GstPadLinkReturn gst_sf_link		(GstPad *pad, GstCaps *caps);
+static GstPadLinkReturn gst_sf_link		(GstPad *pad, const GstCaps *caps);
 
 static void		gst_sf_loop		(GstElement *element);
 
@@ -249,7 +244,7 @@ gst_sfsrc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (sf_src_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&sf_src_factory));
   gst_element_class_set_details (element_class, &sfsrc_details);
 }
 
@@ -258,7 +253,7 @@ gst_sfsink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class, GST_PAD_TEMPLATE_GET (sf_sink_factory));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&sf_sink_factory));
   gst_element_class_set_details (element_class, &sfsink_details);
 }
 
@@ -530,28 +525,27 @@ gst_sf_release_request_pad (GstElement *element, GstPad *pad)
 }
 
 static GstPadLinkReturn
-gst_sf_link (GstPad *pad, GstCaps *caps)
+gst_sf_link (GstPad *pad, const GstCaps *caps)
 {
   GstSF *this = (GstSF*)GST_OBJECT_PARENT (pad);
+  GstStructure *structure;
+
+  structure = gst_caps_get_structure (caps, 0);
   
-  if (GST_CAPS_IS_FIXED (caps)) {
-    gst_caps_get_int (caps, "rate", &this->rate);
-    gst_caps_get_int (caps, "buffer-frames", &this->buffer_frames);
+  gst_structure_get_int  (structure, "rate", &this->rate);
+  gst_structure_get_int  (structure, "buffer-frames", &this->buffer_frames);
 
-    INFO_OBJ (this, "linked pad %s:%s with fixed caps, frames=%d, rate=%d",
-              GST_DEBUG_PAD_NAME (pad), this->rate, this->buffer_frames);
+  INFO_OBJ (this, "linked pad %s:%s with fixed caps, rate=%d, frames=%d",
+            GST_DEBUG_PAD_NAME (pad), this->rate, this->buffer_frames);
 
-    if (this->numchannels) {
-      /* we can go ahead and allocate our buffer */
-      if (this->buffer)
-        g_free (this->buffer);
-      this->buffer = g_malloc (this->numchannels * this->buffer_frames * sizeof (float));
-      memset (this->buffer, 0, this->numchannels * this->buffer_frames * sizeof (float));
-    }
-    return GST_PAD_LINK_OK;
+  if (this->numchannels) {
+    /* we can go ahead and allocate our buffer */
+    if (this->buffer)
+      g_free (this->buffer);
+    this->buffer = g_malloc (this->numchannels * this->buffer_frames * sizeof (float));
+    memset (this->buffer, 0, this->numchannels * this->buffer_frames * sizeof (float));
   }
-
-  return GST_PAD_LINK_DELAYED;
+  return GST_PAD_LINK_OK;
 }
 
 static gboolean
@@ -565,7 +559,9 @@ gst_sf_open_file (GstSF *this)
   this->time = 0;
 
   if (!this->filename) {
-    gst_element_error (GST_ELEMENT (this), "sndfile: 'location' was not set");
+    GST_ELEMENT_ERROR (this, RESOURCE, NOT_FOUND,
+                         (_("No filename specified.")),
+                         (NULL));
     return FALSE;
   }
     
@@ -591,8 +587,8 @@ gst_sf_open_file (GstSF *this)
               this->filename, info.samplerate, info.channels, info.format);
 
     if (!sf_format_check (&info)) {
-      gst_element_error (GST_ELEMENT (this),
-                         g_strdup_printf ("Input parameters (rate:%d, channels:%d, format:0x%x) invalid",
+      GST_ELEMENT_ERROR (this, STREAM, ENCODE, (NULL),
+                           ("Input parameters (rate:%d, channels:%d, format:0x%x) invalid",
                                           info.samplerate, info.channels, info.format));
       return FALSE;
     }
@@ -601,9 +597,9 @@ gst_sf_open_file (GstSF *this)
   this->file = sf_open (this->filename, mode, &info);
 
   if (!this->file) {
-    gst_element_error (GST_ELEMENT (this),
-                       g_strdup_printf ("could not open file \"%s\": %s",
-                                        this->filename, sf_strerror (NULL)));
+    GST_ELEMENT_ERROR (this, RESOURCE, OPEN_WRITE,
+                       (_("Could not open file \"%s\" for writing."), this->filename),
+                         ("soundfile error: %s", sf_strerror (NULL)));
     return FALSE;
   }
 
@@ -640,9 +636,9 @@ gst_sf_close_file (GstSF *this)
   INFO_OBJ (this, "Closing file %s", this->filename);
 
   if ((err = sf_close (this->file)))
-    gst_element_error (GST_ELEMENT (this),
-                       g_strdup_printf ("sndfile: could not close file \"%s\": %s",
-                                        this->filename, sf_error_number (err)));
+    GST_ELEMENT_ERROR (this, RESOURCE, CLOSE,
+                       ("Could not close file file \"%s\".", this->filename),
+                       ("soundfile error: %s", strerror (err)));
   else
     GST_FLAG_UNSET (this, GST_SF_OPEN);
 
@@ -661,7 +657,7 @@ gst_sf_loop (GstElement *element)
   this = (GstSF*)element;
   
   if (this->channels == NULL) {
-    gst_element_error (element, "You must connect at least one pad to sndfile elements.");
+    GST_ELEMENT_ERROR (element, CORE, PAD, (NULL), ("You must connect at least one pad to sndfile elements."));
     return;
   }
 
@@ -702,16 +698,17 @@ gst_sf_loop (GstElement *element)
           continue;
 
         if (!channel->caps_set) {
-          GstCaps *caps = GST_PAD_CAPS (GST_SF_CHANNEL (l)->pad);
+          GstCaps *caps = gst_caps_copy (GST_PAD_CAPS (GST_SF_CHANNEL (l)->pad));
           if (!caps)
             caps = gst_caps_copy
               (GST_PAD_TEMPLATE_CAPS (GST_PAD_PAD_TEMPLATE (GST_SF_CHANNEL (l)->pad)));
-          gst_caps_set (caps, "rate", GST_PROPS_INT (this->rate), NULL);
-          gst_caps_set (caps, "buffer-frames", GST_PROPS_INT (this->buffer_frames), NULL);
+          gst_caps_set_simple (caps,
+              "rate", G_TYPE_INT, this->rate,
+              "buffer-frames", G_TYPE_INT, this->buffer_frames,
+              NULL);
           if (!gst_pad_try_set_caps (GST_SF_CHANNEL (l)->pad, caps)) {
-            gst_element_error (GST_ELEMENT (this),
-                               g_strdup_printf ("Opened file with sample rate %d, but could not set caps",
-                                                this->rate));
+            GST_ELEMENT_ERROR (this, CORE, NEGOTIATION, (NULL),
+                               ("Opened file with sample rate %d, but could not set caps", this->rate));
             gst_sf_close_file (this);
             return;
           }
@@ -759,6 +756,7 @@ gst_sf_loop (GstElement *element)
     for (i=0,l=this->channels; l; l=l->next,i++) {
       channel = GST_SF_CHANNEL (l);
       
+    pull_again:
       in = GST_BUFFER (gst_pad_pull (channel->pad));
 
       if (buffer_frames == 0) {
@@ -766,7 +764,8 @@ gst_sf_loop (GstElement *element)
            which then would set this->buffer_frames to a new value */
         buffer_frames = this->buffer_frames;
         if (buffer_frames == 0) {
-          gst_element_error (element, "Caps were never set, bailing...");
+          GST_ELEMENT_ERROR (element, CORE, NEGOTIATION, (NULL),
+                             ("format wasn't negotiated before chain function"));
           return;
         }
         buf = this->buffer;
@@ -778,20 +777,33 @@ gst_sf_loop (GstElement *element)
           return; /* we've already set gst_element_error */
 
       if (GST_IS_EVENT (in)) {
-        num_to_write = 0;
-      } else {
+        switch (GST_EVENT_TYPE (in)) {
+        case GST_EVENT_EOS:
+        case GST_EVENT_INTERRUPT:
+          num_to_write = 0;
+          break;
+        default:
+          goto pull_again;
+          break;
+        }
+      }
+        
+      if (num_to_write) {
         data = (gfloat*)GST_BUFFER_DATA (in);
         num_to_write = MIN (num_to_write, GST_BUFFER_SIZE (in) / sizeof (gfloat));
         for (j=0; j<num_to_write; j++)
           buf[j * nchannels + i % nchannels] = data[j];
       }
+
       gst_data_unref ((GstData*)in);
     }
 
     if (num_to_write) {
       written = sf_writef_float (this->file, buf, num_to_write);
       if (written != num_to_write)
-        gst_element_error (element, "Error writing file: %s", sf_strerror (this->file));
+        GST_ELEMENT_ERROR (element, RESOURCE, WRITE,
+                             (_("Could not write to file \"%s\"."), this->filename),
+                             ("soundfile error: %s", sf_strerror (this->file)));
     }
 
     this->time += num_to_write * (GST_SECOND / this->rate);
@@ -818,6 +830,11 @@ plugin_init (GstPlugin *plugin)
   if (!gst_element_register (plugin, "sfsink", GST_RANK_NONE, GST_TYPE_SFSINK))
     return FALSE;
 
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return TRUE;
 }
 
