@@ -29,89 +29,158 @@
 GstCaps *
 gst_riff_create_video_caps (guint32             codec_fcc,
 			    gst_riff_strh      *strh,
-			    gst_riff_strf_vids *strf)
+			    gst_riff_strf_vids *strf,
+			    char **codec_name)
 {
   GstCaps *caps = NULL;
 
   switch (codec_fcc) {
     case GST_MAKE_FOURCC('I','4','2','0'):
+      caps = gst_caps_new_simple ("video/x-raw-yuv",
+          "format",  GST_TYPE_FOURCC, codec_fcc,
+          NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Raw, uncompressed I420");
+      break;
     case GST_MAKE_FOURCC('Y','U','Y','2'):
       caps = gst_caps_new_simple ("video/x-raw-yuv",
           "format",  GST_TYPE_FOURCC, codec_fcc,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Raw, uncompressed YUV 4:2:2");
       break;
 
     case GST_MAKE_FOURCC('M','J','P','G'): /* YUY2 MJPEG */
+      caps = gst_caps_new_simple ("video/x-jpeg", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Motion JPEG");
+      break;
     case GST_MAKE_FOURCC('J','P','E','G'): /* generic (mostly RGB) MJPEG */
+      caps = gst_caps_new_simple ("video/x-jpeg", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("JPEG Still Image");
+      break;
+    
     case GST_MAKE_FOURCC('P','I','X','L'): /* Miro/Pinnacle fourccs */
     case GST_MAKE_FOURCC('V','I','X','L'): /* Miro/Pinnacle fourccs */
       caps = gst_caps_new_simple ("video/x-jpeg", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Miro/Pinnacle Video XL");
       break;
-
+    
     case GST_MAKE_FOURCC('H','F','Y','U'):
       caps = gst_caps_new_simple ( "video/x-huffyuv", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Huffman Lossless Codec");
       break;
-
+    
     case GST_MAKE_FOURCC('M','P','E','G'):
     case GST_MAKE_FOURCC('M','P','G','I'):
       caps = gst_caps_new_simple ("video/mpeg",
           "systemstream", G_TYPE_BOOLEAN, FALSE,
           "mpegversion", G_TYPE_BOOLEAN, 1,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG video");
       break;
 
     case GST_MAKE_FOURCC('H','2','6','3'):
+      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("ITU H.26n");
+      break;
     case GST_MAKE_FOURCC('i','2','6','3'):
+      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("ITU H.263");
+      break;
     case GST_MAKE_FOURCC('L','2','6','3'):
+      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Lead H.263");
+      break;
     case GST_MAKE_FOURCC('M','2','6','3'):
+      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft H.263");
+      break;
     case GST_MAKE_FOURCC('V','D','O','W'):
+      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("VDOLive");
+      break;
     case GST_MAKE_FOURCC('V','I','V','O'):
+      caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Vivo H.263");
+      break;
     case GST_MAKE_FOURCC('x','2','6','3'):
       caps = gst_caps_new_simple ("video/x-h263", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Xirlink H.263");
       break;
 
     case GST_MAKE_FOURCC('D','I','V','3'):
-    case GST_MAKE_FOURCC('D','I','V','4'):
-    case GST_MAKE_FOURCC('D','I','V','5'):
       caps = gst_caps_new_simple ("video/x-divx",
           "divxversion", G_TYPE_INT, 3,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("DivX MPEG-4 Version 3");
+      break;
+    case GST_MAKE_FOURCC('D','I','V','4'):
+      caps = gst_caps_new_simple ("video/x-divx",
+          "divxversion", G_TYPE_INT, 4,
+          NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("DivX MPEG-4 Version 4");
       break;
-
     case GST_MAKE_FOURCC('d','i','v','x'):
     case GST_MAKE_FOURCC('D','I','V','X'):
     case GST_MAKE_FOURCC('D','X','5','0'):
+    case GST_MAKE_FOURCC('D','I','V','5'):
       caps = gst_caps_new_simple ("video/x-divx",
           "divxversion", G_TYPE_INT, 5,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("DivX MPEG-4 Version 5");
       break;
 
     case GST_MAKE_FOURCC('X','V','I','D'):
     case GST_MAKE_FOURCC('x','v','i','d'):
       caps = gst_caps_new_simple ("video/x-xvid", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("XVID MPEG-4");
       break;
 
     case GST_MAKE_FOURCC('M','P','G','4'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 41,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft MPEG-4 4.1");
       break;
 
     case GST_MAKE_FOURCC('M','P','4','2'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 42,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft MPEG-4 4.2");
       break;
 
     case GST_MAKE_FOURCC('M','P','4','3'):
       caps = gst_caps_new_simple ("video/x-msmpeg",
           "msmpegversion", G_TYPE_INT, 43,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Microsoft MPEG-4 4.3");
       break;
 
     case GST_MAKE_FOURCC('3','I','V','1'):
     case GST_MAKE_FOURCC('3','I','V','2'):
       caps = gst_caps_new_simple ( "video/x-3ivx", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("3ivx");
       break;
 
     case GST_MAKE_FOURCC('D','V','S','D'):
@@ -119,18 +188,32 @@ gst_riff_create_video_caps (guint32             codec_fcc,
       caps = gst_caps_new_simple ("video/x-dv",
           "systemstream", G_TYPE_BOOLEAN, FALSE,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Generic DV");
       break;
 
     case GST_MAKE_FOURCC('W','M','V','1'):
       caps = gst_caps_new_simple ("video/x-wmv",
           "wmvversion", G_TYPE_INT, 1,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Windows Media Video 7");
       break;
 
     case GST_MAKE_FOURCC('W','M','V','2'):
       caps = gst_caps_new_simple ("video/x-wmv",
           "wmvversion", G_TYPE_INT, 2,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Windows Media Video 8");
+      break;
+    
+    case GST_MAKE_FOURCC('W','M','V','3'):
+      caps = gst_caps_new_simple ("video/x-wmv",
+          "wmvversion", G_TYPE_INT, 3,
+          NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Windows Media Video 9");
       break;
 
     default:
@@ -167,7 +250,8 @@ gst_riff_create_video_caps (guint32             codec_fcc,
 GstCaps *
 gst_riff_create_audio_caps (guint16             codec_id,
 			    gst_riff_strh      *strh,
-			    gst_riff_strf_auds *strf)
+			    gst_riff_strf_auds *strf,
+			    char **codec_name)
 {
   GstCaps *caps = NULL;
 
@@ -177,6 +261,8 @@ gst_riff_create_audio_caps (guint16             codec_id,
           "mpegversion", G_TYPE_INT, 1,
           "layer", G_TYPE_INT, 3,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG 1 layer 3");
       break;
 
     case GST_RIFF_WAVE_FORMAT_MPEGL12: /* mp1 or mp2 */
@@ -184,6 +270,8 @@ gst_riff_create_audio_caps (guint16             codec_id,
           "mpegversion", G_TYPE_INT, 1,
           "layer", G_TYPE_INT, 2,
           NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("MPEG 1 layer 2");
       break;
 
     case GST_RIFF_WAVE_FORMAT_PCM: /* PCM/wav */
@@ -205,6 +293,8 @@ gst_riff_create_audio_caps (guint16             codec_id,
             "width = (int) { 8, 16 }, "
             "height = (int) { 8, 16 }");
       }
+      if (codec_name)
+        *codec_name = g_strdup ("PCM WAV");
       break;
 
     case GST_RIFF_WAVE_FORMAT_MULAW:
@@ -213,6 +303,8 @@ gst_riff_create_audio_caps (guint16             codec_id,
 		     strf->size);
       }
       caps = gst_caps_new_simple ("audio/x-mulaw", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Mulaw");
       break;
 
     case GST_RIFF_WAVE_FORMAT_ALAW:
@@ -221,6 +313,8 @@ gst_riff_create_audio_caps (guint16             codec_id,
 		     strf->size);
       }
       caps = gst_caps_new_simple ("audio/x-alaw", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Alaw");
       break;
 
     case GST_RIFF_WAVE_FORMAT_VORBIS1: /* ogg/vorbis mode 1 */
@@ -230,10 +324,14 @@ gst_riff_create_audio_caps (guint16             codec_id,
     case GST_RIFF_WAVE_FORMAT_VORBIS2PLUS: /* ogg/vorbis mode 2+ */
     case GST_RIFF_WAVE_FORMAT_VORBIS3PLUS: /* ogg/vorbis mode 3+ */
       caps = gst_caps_new_simple ("audio/x-vorbis", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Vorbis");
       break;
 
     case GST_RIFF_WAVE_FORMAT_A52:
       caps = gst_caps_new_simple ("audio/x-ac3", NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("AC3");
       break;
 
     default:
@@ -260,7 +358,8 @@ gst_riff_create_audio_caps (guint16             codec_id,
 GstCaps *
 gst_riff_create_iavs_caps (guint32             codec_fcc,
 			   gst_riff_strh      *strh,
-			   gst_riff_strf_iavs *strf)
+			   gst_riff_strf_iavs *strf,
+			   char **codec_name)
 {
   GstCaps *caps = NULL;
 
@@ -270,6 +369,8 @@ gst_riff_create_iavs_caps (guint32             codec_fcc,
     case GST_MAKE_FOURCC ('d','v','s','d'):
       caps = gst_caps_new_simple ("video/x-dv", 
           "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
+      if (codec_name)
+        *codec_name = g_strdup ("Generic DV");
 
     default:
       GST_WARNING ("Unkown IAVS fourcc " GST_FOURCC_FORMAT,
@@ -312,7 +413,7 @@ gst_riff_create_video_template_caps (void)
 
   caps = gst_caps_new_empty ();
   for (i = 0; tags[i] != 0; i++) {
-    one = gst_riff_create_video_caps (tags[i], NULL, NULL);
+    one = gst_riff_create_video_caps (tags[i], NULL, NULL, NULL);
     if (one)
       gst_caps_append (caps, one);
   }
@@ -339,7 +440,7 @@ gst_riff_create_audio_template_caps (void)
 
   caps = gst_caps_new_empty ();
   for (i = 0; tags[i] != 0; i++) {
-    one = gst_riff_create_audio_caps (tags[i], NULL, NULL);
+    one = gst_riff_create_audio_caps (tags[i], NULL, NULL, NULL);
     if (one)
       gst_caps_append (caps, one);
   }
@@ -360,11 +461,10 @@ gst_riff_create_iavs_template_caps (void)
 
   caps = gst_caps_new_empty ();
   for (i = 0; tags[i] != 0; i++) {
-    one = gst_riff_create_iavs_caps (tags[i], NULL, NULL);
+    one = gst_riff_create_iavs_caps (tags[i], NULL, NULL, NULL);
     if (one)
       gst_caps_append (caps, one);
   }
 
   return caps;
 }
-
