@@ -17,12 +17,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
 #include "mulaw-decode.h"
 #include "mulaw-conversion.h"
 
 extern GstPadTemplate *mulawdec_src_template, *mulawdec_sink_template;
 
+/* elementfactory information */
+static GstElementDetails mulawdec_details = {
+  "Mu Law to PCM conversion",
+  "Codec/Decoder/Audio",
+  "Convert 8bit mu law to 16bit PCM",
+  "Zaheer Merali <zaheer@bellworldwide.net>"
+};
 
 /* Stereo signals and args */
 enum {
@@ -35,12 +45,13 @@ enum {
 };
 
 static void		gst_mulawdec_class_init		(GstMuLawDecClass *klass);
+static void		gst_mulawdec_base_init		(GstMuLawDecClass *klass);
 static void		gst_mulawdec_init			(GstMuLawDec *mulawdec);
 
 static void		gst_mulawdec_set_property			(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void		gst_mulawdec_get_property			(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void		gst_mulawdec_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_mulawdec_chain			(GstPad *pad, GstData *_data);
 
 
 static GstElementClass *parent_class = NULL;
@@ -48,34 +59,29 @@ static GstElementClass *parent_class = NULL;
 
 
 static GstPadLinkReturn
-mulawdec_link (GstPad *pad, GstCaps *caps)
+mulawdec_link (GstPad *pad, const GstCaps *caps)
 {
   GstCaps* tempcaps;
   gint rate, channels;
+  GstStructure *structure;
+  gboolean ret;
   
   GstMuLawDec* mulawdec = GST_MULAWDEC (GST_OBJECT_PARENT (pad));
   
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;  
-  
-  if (!gst_caps_get (caps, "rate", &rate,
-                           "channels", &channels,
-                           NULL))
-    return GST_PAD_LINK_DELAYED;
-  
-  tempcaps = GST_CAPS_NEW (
-	      "sinesrc_src_caps",
-	      "audio/raw",
-          "format",   GST_PROPS_STRING ("int"),
-          "law",      GST_PROPS_INT (0),
-          "depth",    GST_PROPS_INT (16),
-          "width",    GST_PROPS_INT (16),
-          "signed",   GST_PROPS_BOOLEAN (TRUE),
-          "endianness",    GST_PROPS_INT (G_BYTE_ORDER),
-          "rate",     GST_PROPS_INT (rate),
-          "channels", GST_PROPS_INT (channels),
-        NULL);
-  
+  structure = gst_caps_get_structure (caps, 0);
+  ret = gst_structure_get_int (structure, "rate", &rate);
+  ret = gst_structure_get_int (structure, "channels", &channels);
+  if (!ret) return GST_PAD_LINK_REFUSED;
+
+  tempcaps = gst_caps_new_simple ("audio/x-raw-int",
+      "depth",    G_TYPE_INT, 16,
+      "width",    G_TYPE_INT, 16,
+      "signed",   G_TYPE_BOOLEAN, TRUE,
+      "endianness",    G_TYPE_INT, G_BYTE_ORDER,
+      "rate",     G_TYPE_INT, rate,
+      "channels", G_TYPE_INT, channels,
+      NULL);
+
   return gst_pad_try_set_caps (mulawdec->srcpad, tempcaps);
 }
 
@@ -85,7 +91,8 @@ gst_mulawdec_get_type(void) {
 
   if (!mulawdec_type) {
     static const GTypeInfo mulawdec_info = {
-      sizeof(GstMuLawDecClass),      NULL,
+      sizeof(GstMuLawDecClass),
+      (GBaseInitFunc)gst_mulawdec_base_init,
       NULL,
       (GClassInitFunc)gst_mulawdec_class_init,
       NULL,
@@ -99,6 +106,16 @@ gst_mulawdec_get_type(void) {
   return mulawdec_type;
 }
 
+static void
+gst_mulawdec_base_init (GstMuLawDecClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class, mulawdec_src_template);
+  gst_element_class_add_pad_template (element_class, mulawdec_sink_template);
+  gst_element_class_set_details (element_class, &mulawdec_details);
+}
+
 static void
 gst_mulawdec_class_init (GstMuLawDecClass *klass)
 {
@@ -127,8 +144,9 @@ gst_mulawdec_init (GstMuLawDec *mulawdec)
 }
 
 static void
-gst_mulawdec_chain (GstPad *pad,GstBuffer *buf)
+gst_mulawdec_chain (GstPad *pad,GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstMuLawDec *mulawdec;
   gint16 *linear_data;
   guint8 *mulaw_data;
@@ -151,7 +169,7 @@ gst_mulawdec_chain (GstPad *pad,GstBuffer *buf)
   mulaw_decode(mulaw_data,linear_data,GST_BUFFER_SIZE(buf));
 
   gst_buffer_unref(buf);
-  gst_pad_push(mulawdec->srcpad,outbuf);
+  gst_pad_push(mulawdec->srcpad,GST_DATA (outbuf));
 }
 
 static void
