@@ -17,12 +17,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <gst/gst.h>
 #include "mulaw-encode.h"
 #include "mulaw-conversion.h"
 
 extern GstPadTemplate *mulawenc_src_template, *mulawenc_sink_template;
 
+/* elementfactory information */
+static GstElementDetails mulawenc_details = {
+  "PCM to Mu Law conversion",
+  "Codec/Encoder/Audio",
+  "Convert 16bit PCM to 8bit mu law",
+  "Zaheer Merali <zaheer@bellworldwide.net>"
+};
 
 /* Stereo signals and args */
 enum {
@@ -35,47 +45,43 @@ enum {
 };
 
 static void		gst_mulawenc_class_init		(GstMuLawEncClass *klass);
+static void		gst_mulawenc_base_init		(GstMuLawEncClass *klass);
 static void		gst_mulawenc_init			(GstMuLawEnc *mulawenc);
 
 static void		gst_mulawenc_set_property			(GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
 static void		gst_mulawenc_get_property			(GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 
-static void		gst_mulawenc_chain			(GstPad *pad, GstBuffer *buf);
+static void		gst_mulawenc_chain			(GstPad *pad, GstData *_data);
 
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_stereo_signals[LAST_SIGNAL] = { 0 }; */
 
 static GstPadLinkReturn
-mulawenc_link (GstPad *pad, GstCaps *caps)
+mulawenc_link (GstPad *pad, const GstCaps *caps)
 {
   GstCaps* tempcaps;
   gint rate, channels;
+  GstStructure *structure;
+  gboolean ret;
   
   GstMuLawEnc* mulawenc = GST_MULAWENC (GST_OBJECT_PARENT (pad));
   
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;  
+  structure = gst_caps_get_structure (caps, 0);
+  ret = gst_structure_get_int (structure, "rate", &rate);
+  ret = gst_structure_get_int (structure, "channels", &channels);
+  if (!ret) return GST_PAD_LINK_REFUSED;
   
-  if (!gst_caps_get (caps, "rate", &rate,
-                           "channels", &channels,
-                           NULL))
-    return GST_PAD_LINK_DELAYED;
-  
-  tempcaps = GST_CAPS_NEW (
-	      "sinesrc_src_caps",
-	      "audio/raw",
-          "format",   GST_PROPS_STRING ("int"),
-          "law",      GST_PROPS_INT (1),
-          "depth",    GST_PROPS_INT (8),
-          "width",    GST_PROPS_INT (8),
-          "signed",   GST_PROPS_BOOLEAN (FALSE),
-          "rate",     GST_PROPS_INT (rate),
-          "channels", GST_PROPS_INT (channels),
-        NULL);
+  tempcaps = gst_caps_new_simple ("audio/x-mulaw",
+      "depth",    G_TYPE_INT, 8,
+      "width",    G_TYPE_INT, 8,
+      "signed",   G_TYPE_BOOLEAN, FALSE,
+      "rate",     G_TYPE_INT, rate,
+      "channels", G_TYPE_INT, channels,
+      NULL);
   
   return gst_pad_try_set_caps (mulawenc->srcpad, tempcaps);
-}		
+}
 
 GType
 gst_mulawenc_get_type(void) {
@@ -83,7 +89,8 @@ gst_mulawenc_get_type(void) {
 
   if (!mulawenc_type) {
     static const GTypeInfo mulawenc_info = {
-      sizeof(GstMuLawEncClass),      NULL,
+      sizeof(GstMuLawEncClass),
+      (GBaseInitFunc)gst_mulawenc_base_init,
       NULL,
       (GClassInitFunc)gst_mulawenc_class_init,
       NULL,
@@ -97,6 +104,16 @@ gst_mulawenc_get_type(void) {
   return mulawenc_type;
 }
 
+static void
+gst_mulawenc_base_init (GstMuLawEncClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class, mulawenc_src_template);
+  gst_element_class_add_pad_template (element_class, mulawenc_sink_template);
+  gst_element_class_set_details (element_class, &mulawenc_details);
+}
+
 static void
 gst_mulawenc_class_init (GstMuLawEncClass *klass)
 {
@@ -125,8 +142,9 @@ gst_mulawenc_init (GstMuLawEnc *mulawenc)
 }
 
 static void
-gst_mulawenc_chain (GstPad *pad,GstBuffer *buf)
+gst_mulawenc_chain (GstPad *pad,GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   GstMuLawEnc *mulawenc;
   gint16 *linear_data;
   guint8 *mulaw_data;
@@ -149,7 +167,7 @@ gst_mulawenc_chain (GstPad *pad,GstBuffer *buf)
   mulaw_encode(linear_data,mulaw_data,GST_BUFFER_SIZE(outbuf));
 
   gst_buffer_unref(buf);
-  gst_pad_push(mulawenc->srcpad,outbuf);
+  gst_pad_push(mulawenc->srcpad,GST_DATA (outbuf));
 }
 
 static void
