@@ -26,13 +26,10 @@
 /* elementfactory information */
 static GstElementDetails gst_mpegaudio_details = {
   "mpegaudio mp3 encoder",
-  "Codec/Audio/Encoder",
-  "LGPL",
+  "Codec/Encoder/Audio",
   "Uses modified mpegaudio code to encode to mp3 streams",
-  VERSION,
   "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 1999",
+  "Wim Taymans <wim.taymans@chello.be>"
 };
 
 
@@ -52,41 +49,33 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (mpegaudio_sink_templ,
+static GstStaticPadTemplate mpegaudio_sink_templ =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpegaudio_sink_caps",
-    "audio/x-raw-int",
-      "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
-      "signed",      GST_PROPS_BOOLEAN (TRUE),
-      "width",       GST_PROPS_INT (16),
-      "depth",       GST_PROPS_INT (16),
-      "rate",        GST_PROPS_LIST (
-      		       GST_PROPS_INT (32000),
-      		       GST_PROPS_INT (44100),
-      		       GST_PROPS_INT (48000)
-		     ),
-      "channels",    GST_PROPS_LIST (
-                       GST_PROPS_INT (1),
-                       GST_PROPS_INT (2)
-		     )
+  GST_STATIC_CAPS ("audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) true, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) { 32000, 44100, 48000 }, "
+      "channels = (int) [ 1, 2 ]"
   )
-)
+);
 
-GST_PAD_TEMPLATE_FACTORY (mpegaudio_src_templ,
+static GstStaticPadTemplate mpegaudio_src_templ =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mpegaudio_src_caps",
-    "audio/mpeg",
-      "layer",        GST_PROPS_INT_RANGE (1, 2),
-      "channels",     GST_PROPS_INT_RANGE (1, 2),
-      "rate",         GST_PROPS_INT_RANGE (8000, 96000)
+  GST_STATIC_CAPS ("audio/mpeg, "
+      "mpegversion = (int) 1, "
+      "layer = (int) [ 1, 2 ], "
+      "channels = (int) [ 1, 2 ], "
+      "rate = [ 8000, 96000 ]"
   )
-)
+);
 
 /********** Define useful types for non-programmatic interfaces **********/
 #define GST_TYPE_MPEGAUDIO_MODE (gst_mpegaudio_mode_get_type())
@@ -108,6 +97,7 @@ gst_mpegaudio_mode_get_type (void)
 }
 
 static void	gst_mpegaudio_class_init	(GstMpegAudioClass *klass);
+static void	gst_mpegaudio_base_init		(GstMpegAudioClass *klass);
 static void	gst_mpegaudio_init		(GstMpegAudio *mpegaudio);
 
 static void	gst_mpegaudio_chain		(GstPad *pad, GstData *_data);
@@ -125,7 +115,8 @@ gst_mpegaudio_get_type (void)
 
   if (!mpegaudio_type) {
     static const GTypeInfo mpegaudio_info = {
-      sizeof(GstMpegAudioClass),      NULL,
+      sizeof(GstMpegAudioClass),
+      (GBaseInitFunc)gst_mpegaudio_base_init,
       NULL,
       (GClassInitFunc)gst_mpegaudio_class_init,
       NULL,
@@ -139,6 +130,18 @@ gst_mpegaudio_get_type (void)
   return mpegaudio_type;
 }
 
+static void
+gst_mpegaudio_base_init (GstMpegAudioClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+		gst_static_pad_template_get (&mpegaudio_sink_templ));
+  gst_element_class_add_pad_template (element_class,
+		gst_static_pad_template_get (&mpegaudio_src_templ));
+  gst_element_class_set_details (element_class, &gst_mpegaudio_details);
+}
+
 static void
 gst_mpegaudio_class_init (GstMpegAudioClass *klass)
 {
@@ -172,18 +175,18 @@ gst_mpegaudio_class_init (GstMpegAudioClass *klass)
 }
 
 static GstPadLinkReturn
-gst_mpegaudio_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_mpegaudio_sinkconnect (GstPad *pad, const GstCaps *caps)
 {
   GstMpegAudio *mpegaudio;
   gint frequency, channels;
+  GstStructure *structure;
 
   mpegaudio = GST_MPEGAUDIO (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  structure = gst_caps_get_structure (caps, 0);
 
-  gst_caps_get_int (caps, "rate", &frequency);
-  gst_caps_get_int (caps, "channels", &channels);
+  gst_structure_get_int  (structure, "rate", &frequency);
+  gst_structure_get_int  (structure, "channels", &channels);
   mpegaudio->encoder->frequency = frequency;
 
   if (channels == 1)
@@ -191,11 +194,11 @@ gst_mpegaudio_sinkconnect (GstPad *pad, GstCaps *caps)
 
   mpegaudio_sync_parms (mpegaudio->encoder);
 
-  caps = GST_CAPS_NEW ("mpegaudio_src",
-                       "audio/mpeg",
-                         "layer", GST_PROPS_INT (mpegaudio->encoder->info.lay),
-                         "channels", GST_PROPS_INT (channels),
-                         "rate",  GST_PROPS_INT (frequency));
+  caps = gst_caps_new_simple ("audio/mpeg",
+      "mpegversion", G_TYPE_INT, 1,
+      "layer", G_TYPE_INT, mpegaudio->encoder->info.lay,
+      "channels", G_TYPE_INT, channels,
+      "rate",  G_TYPE_INT, frequency, NULL);
 
   return gst_pad_try_set_caps (mpegaudio->srcpad, caps);
 }
@@ -205,13 +208,13 @@ gst_mpegaudio_init (GstMpegAudio *mpegaudio)
 {
   /* create the sink and src pads */
   mpegaudio->sinkpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (mpegaudio_sink_templ), "sink");
+		  gst_static_pad_template_get (&mpegaudio_sink_templ), "sink");
   gst_element_add_pad (GST_ELEMENT (mpegaudio), mpegaudio->sinkpad);
   gst_pad_set_chain_function (mpegaudio->sinkpad, gst_mpegaudio_chain);
   gst_pad_set_link_function (mpegaudio->sinkpad, gst_mpegaudio_sinkconnect);
 
   mpegaudio->srcpad = gst_pad_new_from_template (
-		  GST_PAD_TEMPLATE_GET (mpegaudio_src_templ), "src");
+		  gst_static_pad_template_get (&mpegaudio_src_templ), "src");
   gst_element_add_pad (GST_ELEMENT (mpegaudio), mpegaudio->srcpad);
 
   /* initialize the mpegaudio encoder state */
@@ -353,26 +356,20 @@ gst_mpegaudio_set_property (GObject *object, guint prop_id, const GValue *value,
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the mpegaudio element */
-  factory = gst_element_factory_new ("mpegaudio", GST_TYPE_MPEGAUDIO,
-                                     &gst_mpegaudio_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (mpegaudio_sink_templ));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (mpegaudio_src_templ));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "mpegaudio",
+			       GST_RANK_NONE, GST_TYPE_MPEGAUDIO);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "mpegaudio",
-  plugin_init
-};
+  "MPEG-1 layer 1/2 audio encoder",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
