@@ -32,40 +32,27 @@ static GstElementDetails mp3parse_details = {
   "Erik Walthinsen <omega@cse.ogi.edu>"
 };
 
-static GstPadTemplate*
-mp3_src_factory (void)
-{
-  return
-   gst_pad_template_new (
-  	"src",
-  	GST_PAD_SRC,
-  	GST_PAD_ALWAYS,
-  	gst_caps_new (
-  	  "mp3parse_src",
-    	  "audio/mpeg",
-	  gst_props_new (
-	    "mpegversion", GST_PROPS_INT (1),
-    	    "layer",   GST_PROPS_INT_RANGE (1, 3),
-            "rate",    GST_PROPS_INT_RANGE (8000, 48000),
-            "channels", GST_PROPS_INT_RANGE (1, 2),
-	    NULL)),
-	NULL);
-}
+static GstStaticPadTemplate mp3_src_template =
+GST_STATIC_PAD_TEMPLATE (
+    "src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/mpeg, "
+      "mpegversion = (int) 1, "
+      "layer = (int) [ 1, 3 ], "
+      "rate = (int) [ 8000, 48000], "
+      "channels = (int) [ 1, 2 ]")
+);
 
-static GstPadTemplate*
-mp3_sink_factory (void) 
-{
-  return
-   gst_pad_template_new (
-  	"sink",
-  	GST_PAD_SINK,
-  	GST_PAD_ALWAYS,
-  	gst_caps_new (
-  	  "mp3parse_sink",
-    	  "audio/mpeg",
-	  NULL),
-	NULL);
-};
+static GstStaticPadTemplate mp3_sink_template =
+GST_STATIC_PAD_TEMPLATE (
+    "sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("audio/mpeg, "
+      "mpegversion = (int) 1"
+    )
+);
 
 /* GstMPEGAudioParse signals and args */
 enum {
@@ -80,7 +67,6 @@ enum {
   /* FILL ME */
 };
 
-static GstPadTemplate *sink_temp, *src_temp;
 
 static void	gst_mp3parse_class_init		(GstMPEGAudioParseClass *klass);
 static void	gst_mp3parse_base_init		(GstMPEGAudioParseClass *klass);
@@ -141,61 +127,43 @@ mp3_type_frame_length_from_header (guint32 header, guint *put_layer,
 				   guint *put_samplerate)
 {
   guint length;
-  gulong mode, samplerate, bitrate, layer, version, channels;
+  gulong mode, samplerate, bitrate, layer, channels, padding;
+  gint lsf, mpg25;
 
-  /* we don't need extension, copyright, original or
-   * emphasis for the frame length */
-  header >>= 6;
-
-  /* mode */
-  mode = header & 0x3;
-  header >>= 3;
-
-  /* padding */
-  length = header & 0x1;
-  header >>= 1;
-
-  /* sampling frequency */
-  samplerate = header & 0x3;
-  if (samplerate == 3)
-    return 0;
-  header >>= 2;
-
-  /* bitrate index */
-  bitrate = header & 0xF;
-  if (bitrate == 15 || bitrate == 0)
-    return 0;
-
-  /* ignore error correction, too */
-  header >>= 5;
-
-  /* layer */
-  layer = 4 - (header & 0x3);
-  if (layer == 4)
-    return 0;
-  header >>= 2;
-
-  /* version */
-  version = header & 0x3;
-  if (version == 1)
-    return 0;
-
-  /* lookup */
-  channels = (mode == 3) ? 1 : 2;
-  bitrate = mp3types_bitrates[version == 3 ? 0 : 1][layer - 1][bitrate];
-  samplerate = mp3types_freqs[version > 0 ? version - 1 : 0][samplerate];
-
-  /* calculating */
-  if (layer == 1) {
-    length = ((12000 * bitrate / samplerate) + length) * 4;
+  if (header & (1 << 20)) {
+    lsf = (header & (1 << 19)) ? 0 : 1;
+    mpg25 = 0;
   } else {
-    length += ((layer == 3 && version == 0) ? 144000 : 72000) * bitrate / samplerate;
+    lsf = 1;
+    mpg25 = 1;
+  }
+
+  mode = (header >> 6) & 0x3;
+  channels = (mode == 3) ? 1 : 2;
+  samplerate = (header >> 10) & 0x3;
+  samplerate = mp3types_freqs[lsf + mpg25][samplerate];
+  layer = 4 - ((header >> 17) & 0x3);
+  bitrate = (header >> 12) & 0xF;
+  bitrate = mp3types_bitrates[lsf][layer - 1][bitrate] * 1000;
+  if (bitrate == 0)
+    return 0;
+  padding = (header >> 9) & 0x1;
+  switch (layer) {
+    case 1:
+      length = (bitrate * 12) / samplerate + 4 * padding;
+      break;
+    case 2:
+      length = (bitrate * 144) / samplerate + padding;
+      break;
+    default:
+    case 3:
+      length = (bitrate * 144) / (samplerate << lsf) + padding;
+      break;
   }
 
   GST_DEBUG ("Calculated mp3 frame length of %u bytes", length);
-  GST_DEBUG ("samplerate = %lu - bitrate = %lu - layer = %lu - version = %lu"
-	     " - channels = %lu",
-	     samplerate, bitrate, layer, version, channels);
+  GST_DEBUG ("samplerate = %lu, bitrate = %lu, layer = %lu, channels = %lu",
+	     samplerate, bitrate, layer, channels);
 
   if (put_layer)
     *put_layer = layer;
@@ -246,13 +214,11 @@ mp3_caps_create (guint layer, guint channels,
   g_assert (bitrate);
   g_assert (channels);
 
-  new = GST_CAPS_NEW ("mp3_type_find",
-		      "audio/mpeg",
-			"mpegversion", GST_PROPS_INT (1),
-			"layer",       GST_PROPS_INT (layer),
-			/*"bitrate",     GST_PROPS_INT (bitrate),*/
-			"rate",        GST_PROPS_INT (samplerate),
-			"channels",    GST_PROPS_INT (channels));
+  new = gst_caps_new_simple ("audio/mpeg",
+      "mpegversion", G_TYPE_INT, 1,
+      "layer",       G_TYPE_INT, layer,
+      "rate",        G_TYPE_INT, samplerate,
+      "channels",    G_TYPE_INT, channels, NULL);
 
   return new;
 }
@@ -262,8 +228,10 @@ gst_mp3parse_base_init (GstMPEGAudioParseClass *klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class, sink_temp);
-  gst_element_class_add_pad_template (element_class, src_temp);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mp3_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&mp3_src_template));
   gst_element_class_set_details (element_class, &mp3parse_details);
 }
 
@@ -294,14 +262,17 @@ gst_mp3parse_class_init (GstMPEGAudioParseClass *klass)
 static void
 gst_mp3parse_init (GstMPEGAudioParse *mp3parse)
 {
-  mp3parse->sinkpad = gst_pad_new_from_template(sink_temp, "sink");
+  mp3parse->sinkpad = gst_pad_new_from_template(
+      gst_static_pad_template_get (&mp3_sink_template), "sink");
   gst_element_add_pad(GST_ELEMENT(mp3parse),mp3parse->sinkpad);
 
   gst_pad_set_chain_function(mp3parse->sinkpad,gst_mp3parse_chain);
   gst_element_set_loop_function (GST_ELEMENT(mp3parse),NULL);
 
-  mp3parse->srcpad = gst_pad_new_from_template(src_temp, "src");
+  mp3parse->srcpad = gst_pad_new_from_template(
+      gst_static_pad_template_get (&mp3_src_template), "src");
   gst_element_add_pad(GST_ELEMENT(mp3parse),mp3parse->srcpad);
+  gst_pad_use_explicit_caps (mp3parse->srcpad);
   /*gst_pad_set_type_id(mp3parse->srcpad, mp3frametype); */
 
   mp3parse->partialbuf = NULL;
@@ -345,10 +316,13 @@ gst_mp3parse_chain (GstPad *pad, GstData *_data)
 
   /* if we have something left from the previous frame */
   if (mp3parse->partialbuf) {
+    GstBuffer *newbuf;
 
-    mp3parse->partialbuf = gst_buffer_merge(mp3parse->partialbuf, buf);
+    newbuf = gst_buffer_merge(mp3parse->partialbuf, buf);
     /* and the one we received.. */
     gst_buffer_unref(buf);
+    gst_buffer_unref(mp3parse->partialbuf);
+    mp3parse->partialbuf = newbuf;
   }
   else {
     mp3parse->partialbuf = buf;
@@ -413,6 +387,26 @@ gst_mp3parse_chain (GstPad *pad, GstData *_data)
         GST_DEBUG ("mp3parse: partial buffer needed %ld < %d ",(size-offset), bpf);
 	break;
       } else {
+        guint bitrate, layer, rate, channels;
+
+        if (!mp3_type_frame_length_from_header (header, &layer,
+						&channels,
+						&bitrate, &rate)) {
+	  g_error("Header failed internal error");
+        }
+        if (channels != mp3parse->channels ||
+            rate     != mp3parse->rate     ||
+            layer    != mp3parse->layer    ||
+            bitrate  != mp3parse->bit_rate) {
+          GstCaps *caps = mp3_caps_create (layer, channels, bitrate, rate);
+
+          gst_pad_set_explicit_caps(mp3parse->srcpad, caps);
+
+          mp3parse->channels = channels;
+          mp3parse->layer    = layer;
+          mp3parse->rate     = rate;
+          mp3parse->bit_rate = bitrate;
+        }
 
         outbuf = gst_buffer_create_sub(mp3parse->partialbuf,offset,bpf);
 
@@ -471,23 +465,6 @@ bpf_from_header (GstMPEGAudioParse *parse, unsigned long header)
     return 0;
   }
 
-  if (channels != parse->channels ||
-      rate     != parse->rate     ||
-      layer    != parse->layer    ||
-      bitrate  != parse->bit_rate) {
-    GstCaps *caps = mp3_caps_create (layer, channels, bitrate, rate);
-
-    if (gst_pad_try_set_caps(parse->srcpad, caps) <= 0) {
-      gst_element_error (GST_ELEMENT (parse),
-                         "mp3parse: failed to negotiate format with next element");
-    }
-
-    parse->channels = channels;
-    parse->layer    = layer;
-    parse->rate     = rate;
-    parse->bit_rate = bitrate;
-  }
-
   return length;
 }
 
@@ -585,9 +562,6 @@ gst_mp3parse_change_state (GstElement *element)
 static gboolean
 plugin_init (GstPlugin *plugin)
 {
-  sink_temp = mp3_sink_factory ();
-  src_temp = mp3_src_factory ();
-
   return gst_element_register (plugin, "mp3parse",
 			       GST_RANK_NONE, GST_TYPE_MP3PARSE);
 }
