@@ -210,6 +210,186 @@ gst_qtdemux_init (GstQTDemux *qtdemux)
   gst_element_add_pad (GST_ELEMENT (qtdemux), qtdemux->sinkpad);
 }
 
+static const GstFormat *
+gst_qtdemux_get_src_formats (GstPad *pad)
+{
+  static const GstFormat src_a_formats[] = {
+    GST_FORMAT_TIME,
+    GST_FORMAT_BYTES,
+    GST_FORMAT_DEFAULT,
+    0
+  };
+  static const GstFormat src_v_formats[] = {
+    GST_FORMAT_TIME,
+    GST_FORMAT_DEFAULT,
+    0
+  };
+  QtDemuxStream *stream = gst_pad_get_element_private(pad);
+
+  return (stream->subtype == GST_MAKE_FOURCC('v','i','d','e')) ?
+    src_v_formats : src_a_formats;
+}
+
+static gboolean
+gst_qtdemux_src_convert (GstPad *pad, GstFormat src_format, gint64 src_value,
+    GstFormat *dest_format, gint64 *dest_value)
+{
+  gboolean res = TRUE;
+  QtDemuxStream *stream = gst_pad_get_element_private(pad);
+
+  if (stream->subtype == GST_MAKE_FOURCC('v','i','d','e') &&
+      (src_format == GST_FORMAT_BYTES || *dest_format == GST_FORMAT_BYTES))
+    return FALSE;
+
+  switch (src_format) {
+    case GST_FORMAT_TIME:
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          *dest_value = src_value * 1; /* FIXME */
+          break;
+        case GST_FORMAT_DEFAULT:
+          *dest_value = src_value * 1; /* FIXME */
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    case GST_FORMAT_BYTES:
+      switch (*dest_format) {
+        case GST_FORMAT_TIME:
+          *dest_value = src_value * 1; /* FIXME */
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    case GST_FORMAT_DEFAULT:
+      switch (*dest_format) {
+        case GST_FORMAT_TIME:
+          *dest_value = src_value * 1; /* FIXME */
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    default:
+      res = FALSE;
+  }
+
+  return res;
+}
+
+static const GstQueryType *
+gst_qtdemux_get_src_query_types (GstPad *pad)
+{
+  static const GstQueryType src_types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
+    0
+  };
+
+  return src_types;
+}
+
+static const GstEventMask *
+gst_qtdemux_get_event_mask (GstPad *pad)
+{
+  static const GstEventMask masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_KEY_UNIT },
+    { 0, }
+  };
+
+  return masks;
+}
+
+static gboolean
+gst_qtdemux_handle_src_query (GstPad *pad, GstQueryType type,
+    GstFormat *format, gint64 *value)
+{
+  gboolean res = TRUE;
+  //QtDemuxStream *stream = gst_pad_get_element_private(pad);
+
+  switch (type) {
+    case GST_QUERY_TOTAL:
+      switch (*format) {
+        case GST_FORMAT_TIME:
+          *value = 0; /* FIXME */
+          break;
+        case GST_FORMAT_BYTES:
+          *value = 0; /* FIXME */
+          break;
+        case GST_FORMAT_DEFAULT:
+          *value = 0; /* FIXME */
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    case GST_QUERY_POSITION:
+      switch (*format) {
+        case GST_FORMAT_TIME:
+          *value = 0; /* FIXME */
+          break;
+        case GST_FORMAT_BYTES:
+          *value = 0; /* FIXME */
+          break;
+        case GST_FORMAT_DEFAULT:
+          *value = 0; /* FIXME */
+          break;
+        default:
+          res = FALSE;
+          break;
+      }
+      break;
+    default:
+      res = FALSE;
+      break;
+  }
+
+  return res;
+}
+
+static gboolean
+gst_qtdemux_handle_src_event (GstPad *pad, GstEvent *event)
+{
+  gboolean res = TRUE;
+  //QtDemuxStream *stream = gst_pad_get_element_private(pad);
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_SEEK:
+      GST_DEBUG ("seek format %d", GST_EVENT_SEEK_FORMAT (event));
+
+      switch (GST_EVENT_SEEK_FORMAT (event)) {
+        case GST_FORMAT_BYTES:
+        case GST_FORMAT_DEFAULT:
+        case GST_FORMAT_TIME:
+          {
+            gint64 desired_offset = GST_EVENT_SEEK_OFFSET (event);
+
+            GST_DEBUG ("seeking to %" G_GINT64_FORMAT, desired_offset);
+
+            res = FALSE;
+          }
+        default:
+          res = FALSE;
+          break;
+      }
+    default:
+      res = FALSE;
+      break;
+  }
+
+  gst_event_unref (event);
+
+  return res;
+}
+
+
+
 GST_DEBUG_CATEGORY (qtdemux_debug);
 
 static gboolean
@@ -507,9 +687,10 @@ static void gst_qtdemux_loop_header (GstElement *element)
 void gst_qtdemux_add_stream(GstQTDemux *qtdemux, QtDemuxStream *stream)
 {
   if(stream->subtype == GST_MAKE_FOURCC('v','i','d','e')){
+    gchar *name = g_strdup_printf ("video_%02d", qtdemux->n_video_streams);
     stream->pad = gst_pad_new_from_template (
-	gst_static_pad_template_get(&gst_qtdemux_videosrc_template),
-        g_strdup_printf ("video_%02d", qtdemux->n_video_streams));
+	gst_static_pad_template_get(&gst_qtdemux_videosrc_template), name);
+    g_free (name);
     stream->fps = 1. * GST_SECOND / stream->samples[0].duration;
     if(stream->caps){
       gst_caps_set_simple(stream->caps,
@@ -519,9 +700,10 @@ void gst_qtdemux_add_stream(GstQTDemux *qtdemux, QtDemuxStream *stream)
     }
     qtdemux->n_video_streams++;
   }else{
+    gchar *name = g_strdup_printf ("audio_%02d", qtdemux->n_audio_streams);
     stream->pad = gst_pad_new_from_template (
-	gst_static_pad_template_get(&gst_qtdemux_audiosrc_template),
-        g_strdup_printf ("audio_%02d", qtdemux->n_audio_streams));
+	gst_static_pad_template_get(&gst_qtdemux_audiosrc_template), name);
+    g_free (name);
     if(stream->caps){
       gst_caps_set_simple(stream->caps,
 	  "rate", G_TYPE_INT, (int)stream->rate,
@@ -537,6 +719,13 @@ void gst_qtdemux_add_stream(GstQTDemux *qtdemux, QtDemuxStream *stream)
   qtdemux->n_streams++;
   GST_DEBUG ("n_streams is now %d", qtdemux->n_streams);
 
+  gst_pad_set_event_mask_function (stream->pad, gst_qtdemux_get_event_mask);
+  gst_pad_set_event_function (stream->pad, gst_qtdemux_handle_src_event);
+  gst_pad_set_query_type_function (stream->pad, gst_qtdemux_get_src_query_types);
+  gst_pad_set_query_function (stream->pad, gst_qtdemux_handle_src_query);
+  gst_pad_set_formats_function (stream->pad, gst_qtdemux_get_src_formats);
+  gst_pad_set_convert_function (stream->pad, gst_qtdemux_src_convert);
+
   gst_pad_set_explicit_caps(stream->pad, stream->caps);
 
   GST_DEBUG ("adding pad %p to qtdemux %p", stream->pad, qtdemux);
@@ -1258,7 +1447,9 @@ static void qtdemux_parse_trak(GstQTDemux *qtdemux, GNode *trak)
     
     stream->caps = qtdemux_video_caps(qtdemux,
         QTDEMUX_FOURCC_GET(stsd->data+offset+4), stsd->data);
-    GST_LOG("caps %s\n",gst_caps_to_string(stream->caps));
+    GST_INFO("type " GST_FOURCC_FORMAT " caps %" GST_PTR_FORMAT "\n",
+        GST_FOURCC_ARGS (QTDEMUX_FOURCC_GET(stsd->data+offset+4)),
+        stream->caps);
   }else if(stream->subtype == FOURCC_soun){
     int version;
 
@@ -1293,7 +1484,8 @@ static void qtdemux_parse_trak(GstQTDemux *qtdemux, GNode *trak)
 
     stream->caps = qtdemux_audio_caps(qtdemux,
         QTDEMUX_FOURCC_GET(stsd->data+16+4), (QTDEMUX_GUINT32_GET(stsd->data) > offset) ? stsd->data + offset : NULL);
-    GST_LOG("caps %s\n",gst_caps_to_string(stream->caps));
+    GST_INFO("type " GST_FOURCC_FORMAT " caps %" GST_PTR_FORMAT "\n",
+        GST_FOURCC_ARGS(QTDEMUX_FOURCC_GET(stsd->data+16+4)), stream->caps);
   }else{
     GST_LOG("unknown subtype\n");
     return;
@@ -1410,6 +1602,7 @@ done:
 
       for(j=first_chunk;j<last_chunk;j++){
         int chunk_offset;
+        if(j>=n_samples)goto done2;
         if(stco){
           chunk_offset = QTDEMUX_GUINT32_GET(stco->data + 16 + j*4);
         }else{
@@ -1430,7 +1623,6 @@ done:
 #endif
 	samples[j].sample_index = sample_index;
 	sample_index += samples_per_chunk;
-	if(j>=n_samples)goto done2;
       }
     }
 /*
@@ -1571,9 +1763,10 @@ static GstCaps *qtdemux_video_caps(GstQTDemux *qtdemux, guint32 fourcc, const gu
           "systemstream = (boolean) false");
     case GST_MAKE_FOURCC('3','I','V','1'):
       return gst_caps_from_string ("video/x-3ivx");
-    case GST_MAKE_FOURCC('r','p','z','a'):
     case GST_MAKE_FOURCC('c','v','i','d'):
       /* Cinepak */
+      return gst_caps_from_string ("video/x-cinepak");
+    case GST_MAKE_FOURCC('r','p','z','a'):
     case GST_MAKE_FOURCC('r','l','e',' '):
       /* Run-length encoding */
     case GST_MAKE_FOURCC('s','m','c',' '):
