@@ -29,7 +29,9 @@ This file contains most of the initialisation and control functions
 
 */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -133,7 +135,7 @@ int RTjpeg_b2s(__s16 *data, __s8 *strm, __u8 bt8)
  register int ci, co=1, tmp;
  register __s16 ZZvalue;
 
- (__u8)strm[0]=(__u8)(data[RTjpeg_ZZ[0]]>254) ? 254:((data[RTjpeg_ZZ[0]]<0)?0:data[RTjpeg_ZZ[0]]);
+ strm[0]=(__u8)(data[RTjpeg_ZZ[0]]>254) ? 254:((data[RTjpeg_ZZ[0]]<0)?0:data[RTjpeg_ZZ[0]]);
  
  for(ci=1; ci<=bt8; ci++) 
  {
@@ -2805,7 +2807,7 @@ void RTjpeg_init_mcompress(void)
   fprintf(stderr, "RTjpeg: Could not allocate memory\n");
   exit(-1);
  }
- bzero(RTjpeg_old, ((4*RTjpeg_width*RTjpeg_height)));
+ memset (RTjpeg_old, 0, ((4*RTjpeg_width*RTjpeg_height)));
 }
 
 #ifdef HAVE_LIBMMX
