@@ -31,6 +31,7 @@
 #include <linux/soundcard.h>
 #include <linux/em8300.h>
 
+#include <gst/gst-i18n-plugin.h>
 #include <gst/gst.h>
 
 #include "dxr3audiosink.h"
@@ -48,12 +49,9 @@
 /* ElementFactory information. */
 static GstElementDetails dxr3audiosink_details = {
   "dxr3/Hollywood+ mpeg decoder board audio plugin",
-  "audio/raw|a52",
-  "GPL",
+  "Audio/Sink",
   "Feeds audio to Sigma Designs em8300 based boards",
-  VERSION,
-  "Martin Soto <martinsoto@users.sourceforge.net>",
-  "(C) 2003",
+  "Martin Soto <martinsoto@users.sourceforge.net>"
 };
 
 
@@ -68,49 +66,36 @@ enum {
   ARG_DIGITAL_PCM
 };
 
-
-GST_PAD_TEMPLATE_FACTORY (dxr3audiosink_pcm_sink_factory,
+static GstStaticPadTemplate dxr3audiosink_pcm_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "pcm_sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "dxr3audiosink_pcm_sink",
-    "audio/x-raw-int",
-       "endianness",       GST_PROPS_INT (G_BYTE_ORDER),
-       "signed",           GST_PROPS_BOOLEAN (TRUE),
-       "width",            GST_PROPS_INT (16),
-       "depth",            GST_PROPS_INT (16),
-       "rate",             GST_PROPS_LIST (
-	                     GST_PROPS_INT (32000),
-	                     GST_PROPS_INT (44100),
-	                     GST_PROPS_INT (48000),
-	                     GST_PROPS_INT (66000)
-                           ),
-       "channels",         GST_PROPS_INT (2)
+  GST_STATIC_CAPS (
+    "audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) TRUE, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) { 32000, 44100, 48000, 66000 }, "
+      "channels = (int) 2"
   )
-)
+);
 
-GST_PAD_TEMPLATE_FACTORY (dxr3audiosink_ac3_sink_factory,
+static GstStaticPadTemplate dxr3audiosink_ac3_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "ac3_sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "dxr3audiosink_ac3_sink",
-    "audio/x-ac3",
-    NULL
+  GST_STATIC_CAPS (
+    "audio/x-ac3"
     /* no parameters needed, we don't need a parsed stream */
   )
-)
-
-
-GST_PAD_EVENT_MASK_FUNCTION(dxr3audiosink_get_event_mask,
-  { GST_EVENT_FLUSH, 0 },
-  { GST_EVENT_DISCONTINUOUS, 0 },
-  { GST_EVENT_EOS, 0 }
-)
+);
 
 
 static void	dxr3audiosink_class_init   	(Dxr3AudioSinkClass *klass);
+static void	dxr3audiosink_base_init   	(Dxr3AudioSinkClass *klass);
 static void	dxr3audiosink_init		(Dxr3AudioSink *sink);
 
 static void	dxr3audiosink_set_property	(GObject *object,
@@ -130,7 +115,7 @@ static void	dxr3audiosink_set_clock		(GstElement *element,
                                                  GstClock *clock);
 
 static GstPadLinkReturn	dxr3audiosink_pcm_sinklink (GstPad *pad,
-                                                    GstCaps *caps);
+                                                    const GstCaps *caps);
 static void	dxr3audiosink_set_scr		(Dxr3AudioSink *sink,
                                                  guint32 scr);
 
@@ -160,7 +145,7 @@ dxr3audiosink_get_type (void)
   if (!dxr3audiosink_type) {
     static const GTypeInfo dxr3audiosink_info = {
       sizeof(Dxr3AudioSinkClass),
-      NULL,
+      (GBaseInitFunc) dxr3audiosink_base_init,
       NULL,
       (GClassInitFunc) dxr3audiosink_class_init,
       NULL,
@@ -178,6 +163,19 @@ dxr3audiosink_get_type (void)
 }
 
 
+static void
+dxr3audiosink_base_init (Dxr3AudioSinkClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+	gst_static_pad_template_get (&dxr3audiosink_pcm_sink_factory));
+  gst_element_class_add_pad_template (element_class,
+	gst_static_pad_template_get (&dxr3audiosink_ac3_sink_factory));
+  gst_element_class_set_details (element_class,
+				 &dxr3audiosink_details);
+}
+
 static void
 dxr3audiosink_class_init (Dxr3AudioSinkClass *klass) 
 {
@@ -215,32 +213,22 @@ dxr3audiosink_class_init (Dxr3AudioSinkClass *klass)
 static void 
 dxr3audiosink_init (Dxr3AudioSink *sink) 
 {
-  GstCaps *caps;
   GstPadTemplate *temp;
 
   /* Create the PCM pad. */
-  temp = GST_PAD_TEMPLATE_GET (dxr3audiosink_pcm_sink_factory);
+  temp = gst_static_pad_template_get (&dxr3audiosink_pcm_sink_factory);
   sink->pcm_sinkpad = gst_pad_new_from_template (temp, "pcm_sink");
   gst_pad_set_chain_function (sink->pcm_sinkpad, dxr3audiosink_chain_pcm);
   gst_pad_set_link_function (sink->pcm_sinkpad, dxr3audiosink_pcm_sinklink);
   gst_element_add_pad (GST_ELEMENT (sink), sink->pcm_sinkpad);
 
   /* Create the AC3 pad. */
-  temp = GST_PAD_TEMPLATE_GET (dxr3audiosink_ac3_sink_factory);
+  temp = gst_static_pad_template_get (&dxr3audiosink_ac3_sink_factory);
   sink->ac3_sinkpad = gst_pad_new_from_template (temp, "ac3_sink");
-  caps = gst_pad_template_get_caps (temp);
-  gst_pad_try_set_caps (sink->ac3_sinkpad, caps);
-  gst_caps_unref (caps);
   gst_pad_set_chain_function (sink->ac3_sinkpad, dxr3audiosink_chain_ac3);
   gst_element_add_pad (GST_ELEMENT (sink), sink->ac3_sinkpad);
 
   GST_FLAG_SET (GST_ELEMENT(sink), GST_ELEMENT_EVENT_AWARE);
-  gst_pad_set_event_function (sink->pcm_sinkpad, dxr3audiosink_handle_event);
-  gst_pad_set_event_mask_function (sink->pcm_sinkpad,
-                                   dxr3audiosink_get_event_mask);
-  gst_pad_set_event_function (sink->ac3_sinkpad, dxr3audiosink_handle_event);
-  gst_pad_set_event_mask_function (sink->ac3_sinkpad,
-                                   dxr3audiosink_get_event_mask);
 
   sink->card_number = 0;
 
@@ -322,10 +310,9 @@ dxr3audiosink_open (Dxr3AudioSink *sink)
 
   sink->audio_fd = open (sink->audio_filename, O_WRONLY);
   if (sink->audio_fd < 0) {
-    gst_element_error (GST_ELEMENT (sink),
-                      g_strconcat ("Error opening device file \"",
-                                  sink->audio_filename, "\": ",
-                                  g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE,
+                       (_("Could not open audio device \"%s\" for writing."), sink->audio_filename),
+                        GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -335,10 +322,9 @@ dxr3audiosink_open (Dxr3AudioSink *sink)
 
   sink->control_fd = open (sink->control_filename, O_WRONLY);
   if (sink->control_fd < 0) {
-    gst_element_error (GST_ELEMENT (sink),
-                       g_strconcat ("Error opening device file \"",
-                                    sink->control_filename, "\": ",
-                                    g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE,
+                       (_("Could not open control device \"%s\" for writing."), sink->control_filename),
+                        GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -370,10 +356,9 @@ dxr3audiosink_set_mode_pcm (Dxr3AudioSink *sink)
   tmp = oss_mode;
   if (ioctl (sink->audio_fd, SNDCTL_DSP_SETFMT, &tmp) < 0 ||
       tmp != oss_mode) {
-    gst_element_error (GST_ELEMENT (sink),
-                       g_strconcat ("Error configuring device mode for \"",
-                                    sink->audio_filename, "\": ",
-                                    g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS,
+                       (_("Could not configure audio device \"%s\"."), sink->audio_filename),
+                        GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -385,11 +370,9 @@ dxr3audiosink_set_mode_pcm (Dxr3AudioSink *sink)
   /* Set the sampling rate. */
   tmp = sink->rate;
   if (ioctl (sink->audio_fd, SNDCTL_DSP_SPEED, &tmp) < 0) {
-    gst_element_error (GST_ELEMENT (sink),
-                       g_strconcat ("Error configuring device "
-                                    "sampling rate for \"",
-                                    sink->audio_filename, "\": ",
-                                    g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS,
+                       (_("Could not set audio device \"%s\" to %d Hz."), sink->audio_filename, sink->rate),
+                        GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -424,11 +407,9 @@ dxr3audiosink_set_mode_ac3 (Dxr3AudioSink *sink)
   tmp = AC3_BYTE_RATE;
   if (ioctl (sink->audio_fd, SNDCTL_DSP_SPEED, &tmp) < 0 ||
       tmp != AC3_BYTE_RATE) {
-    gst_element_error (GST_ELEMENT (sink),
-                       g_strconcat ("Error configuring device "
-                                    "sampling rate for \"",
-                                    sink->audio_filename, "\": ",
-                                    g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, SETTINGS,
+                       (_("Could not set audio device \"%s\" to %d Hz."), sink->audio_filename, AC3_BYTE_RATE),
+                        GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -454,18 +435,16 @@ dxr3audiosink_close (Dxr3AudioSink *sink)
   g_return_if_fail (GST_FLAG_IS_SET (sink, DXR3AUDIOSINK_OPEN));
 
   if (close (sink->audio_fd) != 0) {
-    gst_element_error (GST_ELEMENT (sink),
-                       g_strconcat ("Error closing file \"",
-                                    sink->audio_filename, "\": ",
-                                    g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, CLOSE,
+                       (_("Could not close audio device \"%s\"."), sink->audio_filename),
+                        GST_ERROR_SYSTEM);
     return;
   }
 
   if (close (sink->control_fd) != 0) {
-    gst_element_error (GST_ELEMENT (sink),
-                       g_strconcat ("Error closing file \"",
-                                    sink->control_filename, "\": ",
-                                    g_strerror (errno), NULL));
+    GST_ELEMENT_ERROR (sink, RESOURCE, CLOSE,
+                       (_("Could not close control device \"%s\"."), sink->audio_filename),
+                        GST_ERROR_SYSTEM);
     return;
   }
 
@@ -495,30 +474,17 @@ dxr3audiosink_set_clock (GstElement *element, GstClock *clock)
 
 
 static GstPadLinkReturn
-dxr3audiosink_pcm_sinklink (GstPad *pad, GstCaps *caps)
+dxr3audiosink_pcm_sinklink (GstPad *pad, const GstCaps *caps)
 {
   Dxr3AudioSink *sink = DXR3AUDIOSINK (gst_pad_get_parent (pad));
-  const gchar* mimetype;
+  GstStructure *structure = gst_caps_get_structure (caps, 0);
   gint rate;
 
-  if (!GST_CAPS_IS_FIXED(caps)) {
+  if (!gst_caps_is_fixed (caps)) {
     return GST_PAD_LINK_DELAYED;
   }
 
-  mimetype = gst_caps_get_mime(caps);
-  if (strcmp (mimetype, "audio/x-raw-int") != 0) {
-    return GST_PAD_LINK_REFUSED;
-  }
-
-  if (!gst_caps_has_property (caps, "rate")) {
-    return GST_PAD_LINK_REFUSED;
-  }
-
-  gst_caps_get_int (caps, "rate", &rate);
-  if (rate != 32000 && rate != 44100 &&
-      rate != 48000 && rate != 66000) {
-    return GST_PAD_LINK_DELAYED;
-  }
+  gst_structure_get_int (structure, "rate", &rate);
   sink->rate = rate;
 
   return GST_PAD_LINK_OK;
@@ -795,23 +761,3 @@ dxr3audiosink_flushed (Dxr3AudioSink *sink)
 {
   /* Do nothing. */
 }
-
-
-extern gboolean 
-dxr3audiosink_factory_init (GstPlugin *plugin) 
-{ 
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new ("dxr3audiosink",
-                                     GST_TYPE_DXR3AUDIOSINK,
-                                     &dxr3audiosink_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_add_pad_template (factory,
-                     GST_PAD_TEMPLATE_GET (dxr3audiosink_pcm_sink_factory));
-  gst_element_factory_add_pad_template (factory,
-                     GST_PAD_TEMPLATE_GET (dxr3audiosink_ac3_sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
-}
