@@ -257,15 +257,21 @@ gst_osssink_sinkconnect (GstPad *pad, const GstCaps *caps)
   return GST_PAD_LINK_OK;
 }
 
-static inline gint64 
+static inline gint
 gst_osssink_get_delay (GstOssSink *osssink) 
 {
   gint delay = 0;
+  gint ret;
 
   if (GST_OSSELEMENT (osssink)->fd == -1)
     return 0;
 
-  if (ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETODELAY, &delay) < 0) {
+#ifdef SNDCTL_DSP_GETODELAY
+  ret = ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETODELAY, &delay);
+#else
+  ret = -1;
+#endif
+  if (ret < 0) {
     audio_buf_info info;
     if (ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_GETOSPACE, &info) < 0) {
       delay = 0;
@@ -274,6 +280,7 @@ gst_osssink_get_delay (GstOssSink *osssink)
       delay = (info.fragstotal * info.fragsize) - info.bytes;	  
     }
   }
+
   return delay;
 }
 
@@ -289,10 +296,17 @@ gst_osssink_get_time (GstClock *clock, gpointer data)
 
   delay = gst_osssink_get_delay (osssink);
 
-  /* sometimes delay is bigger than the number of bytes sent to the device, 
-   * which screws up this calculation, we assume that everything is still 
-   * in the device then */
-  if (((guint64)delay) > osssink->handled) {
+  /* sometimes delay is bigger than the number of bytes sent to the device,
+   * which screws up this calculation, we assume that everything is still
+   * in the device then
+   * thomas: with proper handling of the return value, this doesn't seem to
+   * happen anymore, so remove the second code path after april 2004 */
+  if (delay < 0) {
+    delay = 0;
+  } else if ((guint64) delay > osssink->handled) {
+    g_warning ("Delay %d > osssink->handled %" G_GUINT64_FORMAT
+               ", setting to osssink->handled",
+               delay, osssink->handled);
     delay = osssink->handled;
   }
   res =  (osssink->handled - delay) * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
@@ -328,6 +342,7 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
   GstClockTime buftime, elementtime, soundtime;
   guchar *data;
   guint to_write;
+  gint delay;
 
   /* this has to be an audio buffer */
   osssink = GST_OSSSINK (gst_pad_get_parent (pad));
@@ -337,7 +352,7 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
 
     switch (GST_EVENT_TYPE (event)) {
       case GST_EVENT_EOS:
-        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_SYNC);
+        ioctl (GST_OSSELEMENT (osssink)->fd, SNDCTL_DSP_SYNC, 0);
 	gst_audio_clock_set_active (GST_AUDIO_CLOCK (osssink->provided_clock), FALSE);
 	gst_pad_event_default (pad, event);
         return;
@@ -358,7 +373,10 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
   to_write = GST_BUFFER_SIZE (buf);
   /* sync audio with buffers timestamp */
   elementtime = gst_element_get_time (GST_ELEMENT (osssink));
-  soundtime = elementtime + gst_osssink_get_delay (osssink) * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
+  delay = gst_osssink_get_delay (osssink);
+  if (delay < 0)
+    delay = 0;
+  soundtime = elementtime + delay * GST_SECOND / GST_OSSELEMENT (osssink)->bps;
   if (GST_BUFFER_TIMESTAMP_IS_VALID (buf)) {
     buftime = GST_BUFFER_TIMESTAMP (buf);
   } else {
@@ -372,9 +390,9 @@ gst_osssink_chain (GstPad *pad, GstData *_data)
 	elementtime, soundtime, buftime);
     if (soundtime > buftime) {
       /* full frames */
-      guint throw_away = (soundtime - buftime) * GST_OSSELEMENT (osssink)->bps / GST_SECOND
-	  / GST_OSSELEMENT (osssink)->width / GST_OSSELEMENT (osssink)->channels
-	  * GST_OSSELEMENT (osssink)->width * GST_OSSELEMENT (osssink)->channels;
+      guint throw_away = (soundtime - buftime) * GST_OSSELEMENT (osssink)->bps / GST_SECOND;
+      throw_away &= ~(GST_OSSELEMENT (osssink)->width * GST_OSSELEMENT (osssink)->channels);
+      osssink->handled += throw_away;
       if (throw_away >= GST_BUFFER_SIZE (buf)) {
 	gst_data_unref (_data);
 	return;
