@@ -99,7 +99,7 @@ gst_v4lmjpegsrc_sync_next_frame (GstV4lMjpegSrc *v4lmjpegsrc,
   while (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd,
                MJPIOC_SYNC, &(v4lmjpegsrc->bsync)) < 0) {
     if (errno != EINTR) {
-      GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SYNC, NULL, GST_ERROR_SYSTEM);
+      GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SYNC, (NULL), GST_ERROR_SYSTEM);
       return FALSE;
     }
     DEBUG("Sync got interrupted");
@@ -161,7 +161,8 @@ gst_v4lmjpegsrc_set_capture (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Query params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_G_PARAMS, &bparm) < 0)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, (NULL),
+                       GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -187,7 +188,8 @@ gst_v4lmjpegsrc_set_capture (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Set params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_S_PARAMS, &bparm) < 0)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, (NULL),
+                       GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -231,7 +233,8 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Query params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_G_PARAMS, &bparm) < 0)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, (NULL),
+                       GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -266,21 +269,21 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
 
   if (width + x_offset > maxwidth)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, (NULL),
       ("Image width+offset (%d) bigger than maximum (%d)",
       width + x_offset, maxwidth));
     return FALSE;
   }
   if ((width%(bparm.HorDcm*16))!=0) 
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, STREAM, FORMAT, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, STREAM, FORMAT, (NULL),
       ("Image width (%d) not multiple of %d (required for JPEG)",
       width, bparm.HorDcm*16));
     return FALSE;
   }
   if (height + y_offset > (norm==VIDEO_MODE_NTSC ? 480 : 576)) 
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, (NULL),
       ("Image height+offset (%d) bigger than maximum (%d)",
       height + y_offset, (norm==VIDEO_MODE_NTSC ? 480 : 576)));
     return FALSE;
@@ -290,7 +293,7 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
    */
   if ((height%(bparm.VerDcm*16))!=0) 
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, STREAM, FORMAT, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, STREAM, FORMAT, (NULL),
       ("Image height (%d) not multiple of %d (required for JPEG)",
       height, bparm.VerDcm*16));
     return FALSE;
@@ -313,7 +316,8 @@ gboolean gst_v4lmjpegsrc_set_capture_m (GstV4lMjpegSrc *v4lmjpegsrc,
   /* Set params for capture */
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd, MJPIOC_S_PARAMS, &bparm) < 0)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, NULL, GST_ERROR_SYSTEM);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, SETTINGS, (NULL),
+                       GST_ERROR_SYSTEM);
     return FALSE;
   }
 
@@ -338,13 +342,13 @@ gst_v4lmjpegsrc_capture_init (GstV4lMjpegSrc *v4lmjpegsrc)
   if (ioctl(GST_V4LELEMENT(v4lmjpegsrc)->video_fd,
             MJPIOC_REQBUFS, &(v4lmjpegsrc->breq)) < 0)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, NULL, GST_ERROR_SYSTEM);
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, (NULL), GST_ERROR_SYSTEM);
     return FALSE;
   }
 
   if (v4lmjpegsrc->breq.count < MIN_BUFFERS_QUEUED)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, READ, (NULL),
       ("Too little buffers. We got %ld, we want at least %d",
       v4lmjpegsrc->breq.count, MIN_BUFFERS_QUEUED));
     return FALSE;
@@ -371,7 +375,7 @@ gst_v4lmjpegsrc_capture_init (GstV4lMjpegSrc *v4lmjpegsrc)
     PROT_READ|PROT_WRITE, MAP_SHARED, GST_V4LELEMENT(v4lmjpegsrc)->video_fd, 0);
   if (GST_V4LELEMENT(v4lmjpegsrc)->buffer == MAP_FAILED)
   {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, (NULL),
       ("Error mapping video buffers: %s", g_strerror (errno)));
     GST_V4LELEMENT(v4lmjpegsrc)->buffer = NULL;
     return FALSE;
@@ -513,7 +517,7 @@ gst_v4lmjpegsrc_requeue_frame (GstV4lMjpegSrc *v4lmjpegsrc,
   g_mutex_lock(v4lmjpegsrc->mutex_queue_state);
 
   if (v4lmjpegsrc->frame_queue_state[num] != QUEUE_STATE_SYNCED) {
-    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, NULL,
+    GST_ELEMENT_ERROR (v4lmjpegsrc, RESOURCE, TOO_LAZY, (NULL),
                       ("Invalid state %d (expected %d), can't requeue",
                       v4lmjpegsrc->frame_queue_state[num],
                       QUEUE_STATE_SYNCED));
